@@ -1,14 +1,14 @@
 #!/usr/bin/env python3
 
 # This file is part of Xpra.
-# Copyright (C) 2011-2013 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 #pylint: disable=bare-except
 
 import datetime
-import subprocess
+from subprocess import Popen, PIPE, STDOUT
 import socket
 import platform
 import os.path
@@ -64,30 +64,31 @@ def save_properties(props, filename):
 
 def get_properties(filename):
     props = dict()
-    if os.path.exists(filename):
-        with open(filename, "rb") as f:
-            for line in f:
-                try:
-                    s = line.decode("utf-8")
-                except UnicodeDecodeError:
-                    #str cannot be decoded!
-                    s = str(line)
-                s = s.strip()
-                if not s:
-                    continue
-                if s[0] in ('!', '#'):
-                    continue
-                parts = s.split("=", 1)
-                if len(parts)<2:
-                    print("missing equal sign: %s" % s)
-                    continue
-                name = parts[0]
-                value = parts[1]
-                if not value:
-                    continue
-                if value[0]!="'" or value[-1]!="'":
-                    continue
-                props[name]= value[1:-1]
+    if not os.path.exists(filename):
+        return props
+    with open(filename, "rb") as f:
+        for line in f:
+            try:
+                s = line.decode("utf-8")
+            except UnicodeDecodeError:
+                #str cannot be decoded!
+                s = str(line)
+            s = s.strip()
+            if not s:
+                continue
+            if s[0] in ('!', '#'):
+                continue
+            parts = s.split("=", 1)
+            if len(parts)<2:
+                print("missing equal sign: %s" % s)
+                continue
+            name = parts[0]
+            value = parts[1]
+            if not value:
+                continue
+            if value[0]!="'" or value[-1]!="'":
+                continue
+            props[name]= value[1:-1]
     return props
 
 
@@ -107,10 +108,10 @@ def get_cpuinfo():
     return "unknown"
 
 def get_status_output(*args, **kwargs):
-    kwargs["stdout"] = subprocess.PIPE
-    kwargs["stderr"] = subprocess.STDOUT
+    kwargs["stdout"] = PIPE
+    kwargs["stderr"] = STDOUT
     try:
-        p = subprocess.Popen(*args, **kwargs)
+        p = Popen(*args, **kwargs)
     except Exception as e:
         print("error running %s,%s: %s" % (args, kwargs, e))
         return -1, "", ""
@@ -191,15 +192,15 @@ def get_platform_name():
         return "Solaris"
     if sys.platform.find("darwin")>=0:
         try:
-            #ie: Mac OS X 10.5.8
-            return "Mac OS X %s" % platform.mac_ver()[0]
-        except:
-            return "Mac OS X"
+            #ie: MacOS 10.14.6
+            return "MacOS %s" % platform.mac_ver()[0]
+        except (AttributeError, TypeError, IndexError):
+            return "MacOS"
     if sys.platform.find("openbsd")>=0:
         return "OpenBSD"
     if sys.platform.startswith("win"):
         try:
-            o = subprocess.Popen('systeminfo', stdout=subprocess.PIPE).communicate()[0]
+            o = Popen('systeminfo', stdout=PIPE).communicate()[0]
             try:
                 o = str(o, "latin-1")  # Python 3+
             except:
@@ -255,7 +256,6 @@ def record_build_info():
     set_prop(props, "COMPILER_VERSION", get_compiler_version())
     set_prop(props, "NVCC_VERSION", get_nvcc_version())
     set_prop(props, "LINKER_VERSION", get_linker_version())
-    set_prop(props, "RELEASE_BUILD", not bool(os.environ.get("BETA", "")))
     #record pkg-config versions:
     PKG_CONFIG = os.environ.get("PKG_CONFIG", "pkg-config")
     for pkg in ("libc",
@@ -280,9 +280,11 @@ def record_build_info():
 
 def get_vcs_props():
     props = {
-                "REVISION" : "unknown",
-                "LOCAL_MODIFICATIONS" : "unknown"
-            }
+        "REVISION" : "unknown",
+        "LOCAL_MODIFICATIONS" : "unknown",
+        "BRANCH" : "unknown",
+        "COMMIT" : "unknown"
+        }
     branch = None
     for cmd in (
         r"git branch --show-current",
@@ -291,7 +293,7 @@ def get_vcs_props():
         #if all else fails:
         r"git branch | grep '* '",
     ):
-        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
+        proc = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
         out, _ = proc.communicate()
         if proc.returncode==0:
             branch_out = out.decode("utf-8").splitlines()
@@ -306,7 +308,7 @@ def get_vcs_props():
         props["BRANCH"] = branch
 
     #use the number of changes since the last tag:
-    proc = subprocess.Popen("git describe --long --always --tags", stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
+    proc = Popen("git describe --long --always --tags", stdout=PIPE, stderr=PIPE, shell=True)
     out, _ = proc.communicate()
     if proc.returncode!=0:
         print("'git describe --long --always --tags' failed with return code %s" % proc.returncode)
@@ -337,8 +339,8 @@ def get_vcs_props():
     if branch=="master":
         #fake a sequential revision number that continues where svn left off,
         #by counting the commits and adding a magic value (5014)
-        proc = subprocess.Popen("git rev-list --count HEAD --first-parent",
-                                stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
+        proc = Popen("git rev-list --count HEAD --first-parent",
+                                stdout=PIPE, stderr=PIPE, shell=True)
         out, _ = proc.communicate()
         if proc.returncode!=0:
             print("failed to get commit count using 'git rev-list --count HEAD'")
@@ -347,7 +349,7 @@ def get_vcs_props():
     props["REVISION"] = rev
     #find number of local files modified:
     changes = 0
-    proc = subprocess.Popen("git status", stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
+    proc = Popen("git status", stdout=PIPE, stderr=PIPE, shell=True)
     (out, _) = proc.communicate()
     if proc.poll()!=0:
         print("could not get status of local files")
@@ -377,7 +379,20 @@ def main(args):
         record_src_info()
     if not has_build_info() or "build" in args:
         record_build_info()
-
+    if "revision" in args:
+        props = get_vcs_props()
+        try:
+            mods = int(props.get("LOCAL_MODIFICATIONS"))
+        except ValueError:
+            mods = 0
+        commit = props.get("COMMIT")
+        print("%s r%s%s%s" % (
+            props.get("BRANCH"),
+            props.get("REVISION"),
+            "M" if mods>0 else "",
+            " (%s)" % commit if commit else "",
+            )
+            )
 
 if __name__ == "__main__":
     main(sys.argv)
