@@ -1,5 +1,11 @@
 #!/bin/bash
 
+if [ -z "${JHBUILD_PREFIX}" ]; then
+	echo "JHBUILD_PREFIX is not set"
+	echo "this script must be executed from jhbuild shell"
+	exit 1
+fi
+
 export PYTHON="python3"
 PYTHON_MAJOR_VERSION=`$PYTHON -c 'import sys;sys.stdout.write("%s" % sys.version_info[0])'`
 PYTHON_MINOR_VERSION=`$PYTHON -c 'import sys;sys.stdout.write("%s" % sys.version_info[1])'`
@@ -72,7 +78,7 @@ fi
 INSTALL_LOG=`pwd`/install.log
 echo "./setup.py install ${BUILD_ARGS}"
 echo " (see ${INSTALL_LOG} for details)"
-${PYTHON} ./setup.py install --old-and-unmanageable ${BUILD_ARGS} >& ${INSTALL_LOG}
+${PYTHON} ./setup.py install --single-version-externally-managed --root=/ --prefix=$JHBUILD_PREFIX ${BUILD_ARGS} >& ${INSTALL_LOG}
 if [ "$?" != "0" ]; then
 	popd
 	echo "ERROR: install failed"
@@ -198,8 +204,8 @@ echo
 echo "*******************************************************************************"
 echo "Add xpra/server/python scripts"
 rsync -pltv ./Helpers/* "${HELPERS_DIR}/"
-#we dont need the wrapper installed by distutils:
-rm ${MACOS_DIR}/Launcher-bin
+#we dont need the wrappers that may have been installed by distutils:
+rm -f ${MACOS_DIR}/*bin
 
 #ensure that every wrapper has a "python" executable to match:
 #(see PythonExecWrapper for why we need this "exec -a" workaround)
@@ -235,7 +241,7 @@ echo "include all xpra modules found: "
 find $PYTHON_PACKAGES/xpra/* -type d -maxdepth 0 -exec basename {} \;
 rsync -rpl $PYTHON_PACKAGES/xpra/* $LIBDIR/python/xpra/
 echo "removing files that should not be installed in the first place (..)"
-for x in "*.c" "*.pyx" "*.pxd" "constants.pxi" "constants.txt"; do
+for x in "*.html" "*.c" "*.cpp" "*.pyx" "*.pxd" "constants.pxi" "constants.txt"; do
 	echo "removing $x:"
 	find $LIBDIR/python/xpra/ -name "$x" -print -exec rm "{}" \; | sed "s+$LIBDIR/python/xpra/++g" | xargs -L 1 echo "* "
 done
@@ -261,7 +267,7 @@ fi
 
 echo
 echo "*******************************************************************************"
-echo "Xpra without a tray..."
+echo "Xpra_NoDock: same app contents but without a dock icon"
 for app in Xpra_NoDock.app; do
 	SUB_APP="${APP_DIR}/Contents/${app}"
 	rsync -rplvtog ${app} ${APP_DIR}/Contents/
@@ -288,10 +294,9 @@ if [ "$STRIP_OPENGL" == "1" ]; then
 	done
 	popd
 fi
-echo " * remove numpy"
-rm -fr $LIBDIR/python/numpy
 pushd $LIBDIR/python
-zip --move -ur site-packages.zip OpenGL
+echo " * zipping OpenGL"
+zip --move -q -r site-packages.zip OpenGL
 popd
 echo " * add gobject-introspection (py2app refuses to do it)"
 rsync -rpl $PYTHON_PACKAGES/gi $LIBDIR/python/
