@@ -12,7 +12,7 @@ import sys
 import unittest
 import binascii
 
-from xpra.os_util import strtobytes, bytestostr, get_util_logger
+from xpra.os_util import strtobytes, bytestostr
 from xpra.util import repr_ellipsized
 from xpra.net.bencode import cython_bencode   #@UnresolvedImport
 from xpra.net import bencode
@@ -312,7 +312,8 @@ class TestFailCython(unittest.TestCase):
         try:
             #backup = sys.modules.copy()
             sys.modules["xpra.net.bencode.cython_bencode"] = None
-            with silence_warn(get_util_logger()):
+            from xpra import os_util
+            with silence_warn(os_util, "util_logger"):
                 bencode.init()
         finally:
             del sys.modules["xpra.net.bencode.cython_bencode"]
