@@ -26,13 +26,15 @@ class ChildCommandMixinTest(ServerMixinTest):
             self.do_test_command_server()
 
     def do_test_command_server(self):
-        from xpra.server.mixins.child_command_server import ChildCommandServer, log
+        from xpra.server.mixins import child_command_server
         opts = AdHocStruct()
         opts.exit_with_children = True
         opts.terminate_children = True
         opts.start_new_commands = True
         opts.start = []
         opts.start_child = []
+        opts.start_late = []
+        opts.start_child_late = []
         opts.start_after_connect = []
         opts.start_child_after_connect = []
         opts.start_on_connect = []
@@ -47,7 +49,7 @@ class ChildCommandMixinTest(ServerMixinTest):
         def noop():
             pass
         def _ChildCommandServer():
-            ccs = ChildCommandServer()
+            ccs = child_command_server.ChildCommandServer()
             ccs.setup_menu_watcher = noop
             return ccs
         self._test_mixin_class(_ChildCommandServer, opts)
@@ -58,7 +60,8 @@ class ChildCommandMixinTest(ServerMixinTest):
         tmpfile = os.path.join(tempfile.gettempdir(), "xpra-test-start-command-%s" % os.getpid())
         assert not os.path.exists(tmpfile)
         command = (b"touch", tmpfile.encode("utf8"))
-        self.handle_packet(("start-command", b"test", command, True))
+        with silence_info(child_command_server):
+            self.handle_packet(("start-command", b"test", command, True))
         time.sleep(1)
         info = self.mixin.get_info(self.protocol)
         commands = info.get("commands")
@@ -70,7 +73,8 @@ class ChildCommandMixinTest(ServerMixinTest):
         assert os.path.exists(tmpfile)
         os.unlink(tmpfile)
         #test signals:
-        self.handle_packet(("start-command", b"sleep", b"sleep 10", True))
+        with silence_info(child_command_server):
+            self.handle_packet(("start-command", b"sleep", b"sleep 10", True))
         time.sleep(1)
         info = self.mixin.get_info(self.protocol)
         commands = info.get("commands")
@@ -82,7 +86,7 @@ class ChildCommandMixinTest(ServerMixinTest):
         assert proc_info.get("name")=="sleep"
         assert proc_info.get("dead") is False
         #send it a SIGINT:
-        with silence_info(log):
+        with silence_info(child_command_server):
             self.handle_packet(("command-signal", pid, "SIGINT"))
         time.sleep(1)
         self.mixin.child_reaper.poll()
