@@ -1,18 +1,18 @@
 BUILD_BIT='64bit'
 BUILD_CPU='x86_64'
-BUILD_DATE='2023-03-10'
+BUILD_DATE='2023-06-20'
 BUILD_MACHINE='x86_64'
 BUILD_OS='Linux Fedora 37 ThirtySeven'
-BUILD_TIME='16:32'
+BUILD_TIME='22:05'
 BUILT_BY='antoine'
 BUILT_ON='localhost.localdomain'
-COMPILER_VERSION='gcc (GCC) 12.2.1 20221121 (Red Hat 12.2.1-4)'
-CYTHON_VERSION='3.0.0a11'
-LINKER_VERSION='GNU ld version 2.38-25.fc37'
-PYTHON_VERSION='3.11.2'
+COMPILER_VERSION='gcc (GCC) 12.3.1 20230508 (Red Hat 12.3.1-1)'
+CYTHON_VERSION='0.29.34'
+LINKER_VERSION='GNU ld version 2.38-27.fc37'
+PYTHON_VERSION='3.11.3'
 lib_gobject_introspection='1.74.0'
-lib_gtk='3.24.37'
-lib_gtk_x11='3.24.37'
+lib_gtk='3.24.38'
+lib_gtk_x11='3.24.38'
 lib_nvenc='10'
 lib_nvfbc='6.0'
 lib_py3cairo='1.21.0'
@@ -20,6 +20,7 @@ lib_pygobject='3.42.2'
 lib_python3='3.11'
 lib_vpx='1.12.0'
 lib_x11='1.8.4'
+lib_x264='0.164.3095'
 lib_x265='3.5'
 lib_xcomposite='0.4.5'
 lib_xdamage='1.1.5'
