@@ -69,6 +69,7 @@ class FakeClient(AdHocStruct):
         self.server_readonly = False
         self.server_pointer = False
         self.update_focus = noop
+        self.has_focus = noop
         self.handle_key_action = noop
         self.window_ungrab = noop
         self.idle_add = no_idle_add
@@ -137,7 +138,7 @@ def test_gl_client_window(gl_client_window_class, max_window_size=(1024, 1024),
         REPAINT_DELAY = envint("XPRA_REPAINT_DELAY", int(show)*16)
         gl_icon = get_icon_filename("opengl", ext="png")
         icon_data = None
-        if os.path.exists(gl_icon):
+        if gl_icon:
             from PIL import Image  # @UnresolvedImport
             img = Image.open(gl_icon)
             img.load()
