@@ -1,25 +1,32 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2022 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os.path
 import sys
+from time import monotonic
 
-from gi.repository import GLib
-from gi.repository import GObject
+from gi.repository import GLib, GObject  # @UnresolvedImport
 
 from xpra.util import (
-    nonl, sorted_nicely, print_nested_dict, envint, flatten_dict, typedict,
-    disconnect_is_an_error, ellipsizer, DONE, first_time,
+    u, net_utf8, nonl, sorted_nicely, print_nested_dict, envint, flatten_dict, typedict,
+    disconnect_is_an_error, ellipsizer, first_time, csv,
+    repr_ellipsized, stderr_write,
+    SERVER_EXIT, DONE,
+    )
+from xpra.os_util import (
+    bytestostr,
+    get_hex_uuid, hexstr,
+    POSIX, OSX,
     )
-from xpra.os_util import bytestostr, strtobytes, get_hex_uuid, POSIX, OSX, hexstr
 from xpra.simple_stats import std_unit
 from xpra.client.client_base import XpraClientBase, EXTRA_TIMEOUT
 from xpra.exit_codes import (
-    EXIT_OK, EXIT_CONNECTION_LOST, EXIT_TIMEOUT, EXIT_INTERNAL_ERROR,
+    EXIT_OK, EXIT_CONNECTION_FAILED, EXIT_TIMEOUT, EXIT_INTERNAL_ERROR,
     EXIT_FAILURE, EXIT_UNSUPPORTED, EXIT_REMOTE_ERROR, EXIT_FILE_TOO_BIG,
+    EXIT_IO_ERROR, EXIT_NO_DATA,
     )
 from xpra.log import Logger
 
@@ -29,11 +36,7 @@ FLATTEN_INFO = envint("XPRA_FLATTEN_INFO", 1)
 
 
 def errwrite(msg):
-    try:
-        sys.stderr.write(msg)
-        sys.stderr.flush()
-    except (OSError, AttributeError):
-        pass
+    stderr_write(msg)
 
 
 class GObjectXpraClient(GObject.GObject, XpraClientBase):
@@ -92,16 +95,19 @@ class GObjectXpraClient(GObject.GObject, XpraClientBase):
 
     def run(self):
         XpraClientBase.run(self)
+        self.run_loop()
+        return self.exit_code
+
+    def run_loop(self):
         self.glib_mainloop = GLib.MainLoop()
         self.glib_mainloop.run()
-        return self.exit_code
 
     def make_hello(self):
         capabilities = XpraClientBase.make_hello(self)
         capabilities["keyboard"] = False
         return capabilities
 
-    def quit(self, exit_code):
+    def quit(self, exit_code=0):
         log("quit(%s) current exit_code=%s", exit_code, self.exit_code)
         if self.exit_code is None:
             self.exit_code = exit_code
@@ -128,11 +134,13 @@ class CommandConnectClient(GObjectXpraClient):
         self.printing = False
         self.command_timeout = None
         #don't bother with many of these things for one-off commands:
-        for x in ("ui_client", "wants_aliases", "wants_encodings",
-                  "wants_versions", "wants_features", "wants_sound", "windows",
-                  "webcam", "keyboard", "mouse", "network-state",
+        for x in ("ui_client",
+                  "wants_aliases", "wants_encodings", "wants_versions", "wants_features", "wants_sound",
+                  "windows", "webcam", "keyboard", "mouse", "network-state",
                   ):
             self.hello_extra[x] = False
+            #for newer versions, it is easier:
+            self.hello_extra["wants"] = []
 
     def setup_connection(self, conn):
         protocol = super().setup_connection(conn)
@@ -149,12 +157,19 @@ class CommandConnectClient(GObjectXpraClient):
             self.command_timeout = None
             GLib.source_remove(ct)
 
-    def _process_connection_lost(self, _packet):
+    def _process_connection_lost(self, packet):
+        log("_process_connection_lost%s", packet)
         #override so we don't log a warning
         #"command clients" are meant to exit quickly by losing the connection
         p = self._protocol
         if p and p.input_packetcount==0:
-            self.quit(EXIT_CONNECTION_LOST)
+            #never got any data back so we have never connected,
+            #try to give feedback to the user as to why that is:
+            details = ""
+            if len(packet)>1:
+                details = ": %s" % csv(packet[1:])
+            log.warn("Connection failed%s", details)
+            self.quit(EXIT_CONNECTION_FAILED)
         else:
             self.quit(EXIT_OK)
 
@@ -239,10 +254,11 @@ class ScreenshotXpraClient(CommandConnectClient):
 
     def _process_screenshot(self, packet):
         (w, h, encoding, _, img_data) = packet[1:6]
-        assert encoding==b"png", "expected png screenshot data but got %s" % bytestostr(encoding)
+        assert bytestostr(encoding)=="png", "expected png screenshot data but got %s" % bytestostr(encoding)
         if not img_data:
             self.warn_and_quit(EXIT_OK,
-                               "screenshot is empty and has not been saved (maybe there are no windows or they are not currently shown)")
+                               "screenshot is empty and has not been saved"+
+                               " (maybe there are no windows or they are not currently shown)")
             return
         if self.screenshot_filename=="-":
             output = os.fdopen(sys.stdout.fileno(), "wb", closefd=False)
@@ -251,7 +267,7 @@ class ScreenshotXpraClient(CommandConnectClient):
         with output:
             output.write(img_data)
             output.flush()
-        self.warn_and_quit(EXIT_OK, "screenshot %sx%s saved to: %s" % (w, h, self.screenshot_filename))
+        self.warn_and_quit(EXIT_OK, f"screenshot {w}x{h} saved to: {self.screenshot_filename!r}")
 
     def init_packet_handlers(self):
         super().init_packet_handlers()
@@ -274,7 +290,13 @@ class InfoXpraClient(CommandConnectClient):
         self.warn_and_quit(EXIT_TIMEOUT, "timeout: did not receive the info")
 
     def do_command(self, caps : typedict):
-        if caps:
+        def print_fn(s):
+            sys.stdout.write(f"{s}\n")
+        if not caps:
+            self.quit(EXIT_NO_DATA)
+            return
+        exit_code = EXIT_OK
+        try:
             if FLATTEN_INFO<2:
                 #compatibility mode:
                 c = flatten_dict(caps)
@@ -289,19 +311,18 @@ class InfoXpraClient(CommandConnectClient):
                         if isinstance(w, bytes):
                             if k.endswith(".data"):
                                 return hexstr(w)
-                            try:
-                                return w.decode("utf-8")
-                            except:
-                                return bytestostr(w)
+                            return u(w)
                         elif isinstance(w, (tuple,list)):
                             return type(w)([fixvalue(x) for x in w])
                         return w
                     v = fixvalue(v)
                     k = fixvalue(k)
-                    log.info("%s=%s", k, nonl(v))
+                    print_fn(f"{k}={nonl(v)}")
             else:
-                print_nested_dict(caps)
-        self.quit(EXIT_OK)
+                print_nested_dict(caps, print_fn=print_fn)
+        except OSError:
+            exit_code = EXIT_IO_ERROR
+        self.quit(exit_code)
 
 class IDXpraClient(InfoXpraClient):
 
@@ -310,6 +331,16 @@ class IDXpraClient(InfoXpraClient):
         self.hello_extra["request"] = "id"
 
 
+class RequestXpraClient(CommandConnectClient):
+
+    def __init__(self, request, opts):
+        super().__init__(opts)
+        self.hello_extra["request"] = request
+
+    def do_command(self, caps : typedict):
+        self.quit(EXIT_OK)
+
+
 class ConnectTestXpraClient(CommandConnectClient):
     """ This client does one thing only:
         it queries the server with an 'info' request
@@ -358,8 +389,9 @@ class MonitorXpraClient(SendCommandConnectClient):
 
     def __init__(self, opts):
         super().__init__(opts)
-        for x in ("wants_features", "wants_events", "event_request"):
-            self.hello_extra[x] = True
+        for x in ("features", "events", "request"):
+            self.hello_extra[f"wants_{x}"] = True
+            self.hello_extra.setdefault("wants", []).append(x)
         self.hello_extra["request"] = "event"
         self.hello_extra["info-namespace"] = True
 
@@ -383,6 +415,92 @@ class MonitorXpraClient(SendCommandConnectClient):
         self.send("ping_echo", echotime, 0, 0, 0, -1)
 
 
+class InfoTimerClient(MonitorXpraClient):
+    """
+        This client keeps monitoring the server
+        and requesting info data
+    """
+    REFRESH_RATE = envint("XPRA_REFRESH_RATE", 1)
+
+    def __init__(self, *args):
+        super().__init__(*args)
+        self.info_request_pending = False
+        self.server_last_info = typedict()
+        self.server_last_info_time = 0
+        self.info_timer = 0
+
+    def run(self):
+        from xpra.gtk_common.gobject_compat import register_os_signals
+        register_os_signals(self.signal_handler, None)
+        v = super().run()
+        self.log("run()=%s" % v)
+        self.cleanup()
+        return v
+
+    def signal_handler(self, signum, *args):
+        self.log("exit_code=%s" % self.exit_code)
+        self.log("signal_handler(%s, %s)" % (signum, args,))
+        self.quit(128+signum)
+        self.log("exit_code=%s" % self.exit_code)
+
+    def log(self, message):
+        #this method is overriden in top client to use a log file
+        log(message)
+
+    def err(self, e):
+        log.error(str(e))
+
+    def cleanup(self):
+        self.cancel_info_timer()
+        MonitorXpraClient.cleanup(self)
+
+    def do_command(self, caps : typedict):
+        self.send_info_request()
+        self.timeout_add(self.REFRESH_RATE*1000, self.send_info_request)
+
+    def send_info_request(self, *categories):
+        self.log("send_info_request%s" % (categories,))
+        if not self.info_request_pending:
+            self.info_request_pending = True
+            window_ids = ()    #no longer used or supported by servers
+            self.send("info-request", [self.uuid], window_ids, categories)
+        if not self.info_timer:
+            self.info_timer = self.timeout_add((self.REFRESH_RATE+2)*1000, self.info_timeout)
+        return True
+
+    def init_packet_handlers(self):
+        MonitorXpraClient.init_packet_handlers(self)
+        self.add_packet_handler("info-response", self._process_info_response, False)
+
+    def _process_server_event(self, packet):
+        self.log("server event: %s" % (packet,))
+        self.last_server_event = packet[1:]
+        self.update_screen()
+
+    def _process_info_response(self, packet):
+        self.log("info response: %s" % repr_ellipsized(packet))
+        self.cancel_info_timer()
+        self.info_request_pending = False
+        self.server_last_info = typedict(packet[1])
+        self.server_last_info_time = monotonic()
+        #log.info("server_last_info=%s", self.server_last_info)
+        self.update_screen()
+
+    def cancel_info_timer(self):
+        it = self.info_timer
+        if it:
+            self.info_timer = None
+            self.source_remove(it)
+
+    def info_timeout(self):
+        self.log("info timeout")
+        self.update_screen()
+        return True
+
+    def update_screen(self):
+        raise NotImplementedError()
+
+
 class ShellXpraClient(SendCommandConnectClient):
     """
         Provides an interactive shell with the socket it connects to
@@ -395,7 +513,11 @@ class ShellXpraClient(SendCommandConnectClient):
         self.hello_extra["shell"] = "True"
 
     def timeout(self, *args):
-        pass
+        """
+        The shell client never times out,
+        but the superclass calls this method automatically,
+        just ignore it.
+        """
 
     def cleanup(self):
         siw = self.stdin_io_watch
@@ -459,7 +581,7 @@ class ShellXpraClient(SendCommandConnectClient):
             stream = sys.stderr
         else:
             raise Exception("invalid file descriptor %i" % fd)
-        s = message.decode("utf8")
+        s = net_utf8(message)
         if s.endswith("\n"):
             s = s[:-1]
         stream.write("%s" % s)
@@ -490,7 +612,7 @@ class VersionXpraClient(HelloRequestClient):
         return True
 
     def do_command(self, caps : typedict):
-        v = caps.strget(b"version")
+        v = caps.strget("version")
         if not v:
             self.warn_and_quit(EXIT_FAILURE, "server did not provide the version information")
         else:
@@ -524,12 +646,7 @@ class ControlXpraClient(CommandConnectClient):
     def make_hello(self):
         capabilities = super().make_hello()
         log("make_hello() adding command request '%s' to %s", self.command, capabilities)
-        def b(s):
-            try:
-                return s.encode("utf8")
-            except Exception:
-                return strtobytes(s)
-        capabilities["command_request"] = tuple(b(x) for x in self.command)
+        capabilities["command_request"] = tuple(self.command)
         capabilities["request"] = "command"
         return capabilities
 
@@ -553,7 +670,7 @@ class PrintClient(SendCommandConnectClient):
             #replace with filename proposed
             self.filename = command[2]
             #read file from stdin
-            with open(sys.stdin.fileno(), mode='rb', closefd=False) as stdin_binary:
+            with open(sys.stdin.fileno(), mode="rb", closefd=False) as stdin_binary:
                 self.file_data = stdin_binary.read()
             log("read %i bytes from stdin", len(self.file_data))
         else:
@@ -581,8 +698,9 @@ class PrintClient(SendCommandConnectClient):
         if not printing:
             self.warn_and_quit(EXIT_UNSUPPORTED, "server does not support printing")
             return
-        #TODO: compress file data? (this should run locally most of the time anyway)
-        from xpra.net.compression import Compressed
+        #we don't compress file data
+        #(this should run locally most of the time anyway)
+        from xpra.net.compression import Compressed  #pylint: disable=import-outside-toplevel
         blob = Compressed("print", self.file_data)
         self.send("print", self.filename, blob, *self.command)
         log("print: sending %s as %s for printing", self.filename, blob)
@@ -590,6 +708,7 @@ class PrintClient(SendCommandConnectClient):
 
     def make_hello(self):
         capabilities = super().make_hello()
+        capabilities.setdefault("wants", []).append("features")
         capabilities["wants_features"] = True   #so we know if printing is supported or not
         capabilities["print_request"] = True    #marker to skip full setup
         capabilities["request"] = "print"
@@ -609,7 +728,7 @@ class ExitXpraClient(HelloRequestClient):
             }
 
     def do_command(self, caps : typedict):
-        self.idle_add(self.send, "exit-server")
+        self.idle_add(self.send, "exit-server", os.environ.get("XPRA_EXIT_MESSAGE", SERVER_EXIT))
 
 
 class StopXpraClient(HelloRequestClient):
@@ -694,7 +813,7 @@ class RequestStartClient(HelloRequestClient):
                 }.get(mode, "")
             try:
                 errwrite("\n%ssession now available on display %s\n" % (session_type, display))
-                if POSIX and not OSX and self.displayfd>0 and display and display.startswith(b":"):
+                if POSIX and not OSX and self.displayfd>0 and display and display.startswith(":"):
                     from xpra.platform.displayfd import write_displayfd
                     log("writing display %s to displayfd=%s", display, self.displayfd)
                     write_displayfd(self.displayfd, display[1:])
