@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # Copyright (C) 2017-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -7,11 +7,11 @@ from xpra.platform import program_context
 from xpra.platform.gui import force_focus
 from xpra.gtk_common.gtk_util import add_close_accel, get_icon_pixbuf
 
-import cairo
+from cairo import OPERATOR_CLEAR, OPERATOR_SOURCE  #pylint: disable=no-name-in-module
 import gi
-gi.require_version("Gtk", "3.0")
-gi.require_version("Gdk", "3.0")
-from gi.repository import Gtk, Gdk, GLib
+gi.require_version("Gtk", "3.0")  # @UndefinedVariable
+gi.require_version("Gdk", "3.0")  # @UndefinedVariable
+from gi.repository import Gtk, Gdk, GLib  # @UnresolvedImport
 
 
 
@@ -27,7 +27,7 @@ class ColorGradientWindow(Gtk.Window):
         if icon:
             self.set_icon(icon)
         self.bpc = 16
-        self.set_title("Color Bit Depth: %i" % self.bpc)
+        self.update_title()
         drawing_area = Gtk.DrawingArea()
         drawing_area.connect("draw", self.area_draw)
         self.add(drawing_area)
@@ -37,6 +37,9 @@ class ColorGradientWindow(Gtk.Window):
         self.connect("key_press_event", self.on_key_press)
         self.connect("button-press-event", self.on_button_press)
 
+    def update_title(self):
+        self.set_title("Color Bit Depth: %i" % self.bpc)
+
     def show_with_focus(self):
         force_focus()
         self.show_all()
@@ -45,7 +48,7 @@ class ColorGradientWindow(Gtk.Window):
     def configure_event(self, *_args):
         self.queue_draw()
 
-    def on_button_press(self, widget, event):
+    def on_button_press(self, _widget, event):
         if event.type!=Gdk.EventType.BUTTON_PRESS:
             return
         if event.button==1:
@@ -53,7 +56,7 @@ class ColorGradientWindow(Gtk.Window):
         else:
             self.bpc -= 1
         self.bpc = (self.bpc+16) % 16
-        self.set_title("Color Bit Depth: %i" % self.bpc)
+        self.update_title()
         self.queue_draw()
         return True
 
@@ -62,7 +65,7 @@ class ColorGradientWindow(Gtk.Window):
             self.bpc = ((self.bpc-2) % 16)+1
         else:
             self.bpc = (self.bpc%16)+1
-        self.set_title("Color Bit Depth: %i" % self.bpc)
+        self.update_title()
         self.queue_draw()
         return True
 
@@ -73,7 +76,7 @@ class ColorGradientWindow(Gtk.Window):
 
     def area_draw(self, widget, cr):
         cr.save()
-        cr.set_operator(cairo.OPERATOR_CLEAR)
+        cr.set_operator(OPERATOR_CLEAR)
         alloc = widget.get_allocated_size()[0]
         w, h = alloc.width, alloc.height
         cr.rectangle(0, 0, w, h)
@@ -94,7 +97,7 @@ class ColorGradientWindow(Gtk.Window):
         def paint_block(R=M, G=M, B=M, label=""):
             y = h*self.index//blocks
             self.index += 1
-            cr.set_operator(cairo.OPERATOR_SOURCE)
+            cr.set_operator(OPERATOR_SOURCE)
             for i in range(w):
                 v = i/w
                 cr.save()
