@@ -1,9 +1,8 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import os
 from time import monotonic
 
 from xpra.exit_codes import EXIT_INTERNAL_ERROR
@@ -16,8 +15,16 @@ from xpra.platform.gui import (
 from xpra.scripts.main import check_display
 from xpra.scripts.config import FALSE_OPTIONS
 from xpra.net.common import MAX_PACKET_SIZE
+from xpra.common import (
+    adjust_monitor_refresh_rate, get_refresh_rate_for_value,
+    FULL_INFO,
+    )
+from xpra.client.scaling_parser import (
+    parse_scaling, scaleup_value, scaledown_value, fequ, r4cmp,
+    MIN_SCALING, MAX_SCALING, SCALING_EMBARGO_TIME,
+    )
 from xpra.util import (
-    envint, envfloat, envbool, log_screen_sizes, flatten_dict, typedict,
+    envint, envbool, log_screen_sizes, flatten_dict, typedict,
     XPRA_SCALING_NOTIFICATION_ID,
     )
 from xpra.client.mixins.stub_client_mixin import StubClientMixin
@@ -29,19 +36,9 @@ scalinglog = Logger("scaling")
 
 MONITOR_CHANGE_REINIT = envint("XPRA_MONITOR_CHANGE_REINIT")
 
-
-MIN_SCALING = envfloat("XPRA_MIN_SCALING", "0.1")
-MAX_SCALING = envfloat("XPRA_MAX_SCALING", "8")
-SCALING_OPTIONS = [float(x) for x in os.environ.get("XPRA_TRAY_SCALING_OPTIONS", "0.25,0.5,0.666,1,1.25,1.5,2.0,3.0,4.0,5.0").split(",") if float(x)>=MIN_SCALING and float(x)<=MAX_SCALING]
-SCALING_EMBARGO_TIME = int(os.environ.get("XPRA_SCALING_EMBARGO_TIME", "1000"))/1000.0
 SYNC_ICC = envbool("XPRA_SYNC_ICC", True)
 
 
-def r4cmp(v, rounding=1000.0):    #ignore small differences in floats for scale values
-    return round(v*rounding)
-def fequ(v1, v2):
-    return r4cmp(v1)==r4cmp(v2)
-
 
 class DisplayClient(StubClientMixin):
     """
@@ -58,6 +55,7 @@ class DisplayClient(StubClientMixin):
         self.initial_scaling = 1, 1
         self.xscale, self.yscale = self.initial_scaling
         self.scale_change_embargo = float("inf")
+        self.refresh_rate = 0
         self.desktop_fullscreen = False
         self.desktop_scaling = False
         self.screen_size_change_timer = None
@@ -68,11 +66,14 @@ class DisplayClient(StubClientMixin):
         self.server_display = None
         self.server_randr = False
         self.server_opengl = None
+        self.server_multi_monitors = False
+        self.server_monitors = {}
 
 
     def init(self, opts):
         self.desktop_fullscreen = opts.desktop_fullscreen
         self.desktop_scaling = opts.desktop_scaling
+        self.refresh_rate = opts.refresh_rate
         self.dpi = int(opts.dpi)
         self.can_scale = opts.desktop_scaling not in FALSE_OPTIONS
         scalinglog("can_scale(%s)=%s", opts.desktop_scaling, self.can_scale)
@@ -81,7 +82,6 @@ class DisplayClient(StubClientMixin):
 
     def parse_scaling(self, desktop_scaling):
         root_w, root_h = self.get_root_size()
-        from xpra.client.scaling_parser import parse_scaling
         self.initial_scaling = parse_scaling(desktop_scaling, root_w, root_h, MIN_SCALING, MAX_SCALING)
         self.xscale, self.yscale = self.initial_scaling
         scalinglog("scaling(%s)=%s", self.initial_scaling, (self.xscale, self.yscale))
@@ -117,9 +117,10 @@ class DisplayClient(StubClientMixin):
             "show-desktop"  : True,
             "vrefresh"      : self.get_vrefresh(),
             }
-        wm_name = get_wm_name()
-        if wm_name:
-            caps["wm_name"] = wm_name
+        if FULL_INFO>0:
+            wm_name = get_wm_name()
+            if wm_name:
+                caps["wm_name"] = wm_name
 
         self._last_screen_settings = self.get_screen_settings()
         root_w, root_h, sss, ndesktops, desktop_names, u_root_w, u_root_h = self._last_screen_settings[:7]
@@ -150,7 +151,8 @@ class DisplayClient(StubClientMixin):
             root_w, root_h = u_root_w, u_root_h
             sss = ss
         caps["screen_sizes"] = sss
-
+        monitors = self.get_monitors_info()
+        caps["monitors"] = adjust_monitor_refresh_rate(self.refresh_rate, monitors)
         caps.update(self.get_screen_caps())
         caps.update(flatten_dict({
             "dpi"               : self.get_dpi_caps(),
@@ -197,11 +199,13 @@ class DisplayClient(StubClientMixin):
     def get_screen_caps(self) -> dict:
         caps = {
             "vrefresh"      : self.get_vrefresh(),
-            "antialias"    : get_antialias_info(),
             "cursor" : {
                 "size"  : int(2*get_cursor_size()/(self.xscale+self.yscale)),
                 },
             }
+        aa = get_antialias_info()
+        if aa:
+            caps["antialias"] = aa
         if SYNC_ICC:
             caps.update({
             "icc"          : self.get_icc_info(),
@@ -246,35 +250,42 @@ class DisplayClient(StubClientMixin):
         self.server_randr = c.boolget("resize_screen")
         log("server has randr: %s", self.server_randr)
         self.server_opengl = c.dictget("opengl")
+        Logger("screen", "opengl")("server opengl=%s", self.server_opengl)
+        self.server_multi_monitors = c.boolget("multi-monitors", False)
+        self.server_monitors = c.dictget("monitors")
+        log("server multi-monitors=%s, monitors=%s",
+                              self.server_multi_monitors, self.server_monitors)
         return True
 
     def process_ui_capabilities(self, c : typedict):
-        self.server_is_desktop = c.boolget("shadow") or c.boolget("desktop")
-        skip_vfb_size_check = False           #if we decide not to use scaling, skip warnings
-        if not fequ(self.xscale, 1.0) or not fequ(self.yscale, 1.0):
-            #scaling is used, make sure that we need it and that the server can support it
-            #(without rounding support, size-hints can cause resize loops)
-            if self.server_is_desktop and not self.desktop_fullscreen:
-                #don't honour auto mode in this case
-                if self.desktop_scaling=="auto":
-                    log.info(" not scaling a %s server", c.strget("type", "shadow"))
-                    skip_vfb_size_check = self.xscale>1 or self.yscale>1
-                    self.scale_change_embargo = 0
-                    self.scalingoff()
-        if self.can_scale:
-            self.may_adjust_scaling()
-        if not self.server_is_desktop and not skip_vfb_size_check and self.server_max_desktop_size:
-            avail_w, avail_h = self.server_max_desktop_size
-            root_w, root_h = self.get_root_size()
-            log("validating server_max_desktop_size=%s vs root size=%s",
-                self.server_max_desktop_size, (root_w, root_h))
-            if self.cx(root_w)!=root_w or self.cy(root_h)!=root_h:
-                log(" root size scaled to %s", (self.cx(root_w), self.cy(root_h)))
-            if self.cx(root_w)>(avail_w+1) or self.cy(root_h)>(avail_h+1):
-                log.warn("Server's virtual screen is too small")
-                log.warn(" server: %sx%s vs client: %sx%s", avail_w, avail_h, self.cx(root_w), self.cy(root_h))
-                log.warn(" you may see strange behavior,")
-                log.warn(" please see https://github.com/Xpra-org/xpra/blob/master/docs/Usage/Xdummy.md")
+        self.server_is_desktop = c.boolget("shadow") or c.boolget("desktop") or c.boolget("monitor")
+        log("process_ui_capabilities(%s) desktop=%s, monitor=%s", c, c.boolget("desktop"), c.boolget("monitor"))
+        if not c.boolget("monitor"):
+            skip_vfb_size_check = False           #if we decide not to use scaling, skip warnings
+            if not fequ(self.xscale, 1.0) or not fequ(self.yscale, 1.0):
+                #scaling is used, make sure that we need it and that the server can support it
+                #(without rounding support, size-hints can cause resize loops)
+                if self.server_is_desktop and not self.desktop_fullscreen:
+                    #don't honour auto mode in this case
+                    if self.desktop_scaling=="auto":
+                        log.info(" not scaling a %s server", c.strget("type", "shadow"))
+                        skip_vfb_size_check = self.xscale>1 or self.yscale>1
+                        self.scale_change_embargo = 0
+                        self.scalingoff()
+            if self.can_scale:
+                self.may_adjust_scaling()
+            if not self.server_is_desktop and not skip_vfb_size_check and self.server_max_desktop_size:
+                avail_w, avail_h = self.server_max_desktop_size
+                root_w, root_h = self.get_root_size()
+                log("validating server_max_desktop_size=%s vs root size=%s",
+                    self.server_max_desktop_size, (root_w, root_h))
+                if self.cx(root_w)!=root_w or self.cy(root_h)!=root_h:
+                    log(" root size scaled to %s", (self.cx(root_w), self.cy(root_h)))
+                if self.cx(root_w)>(avail_w+1) or self.cy(root_h)>(avail_h+1):
+                    log.warn("Server's virtual screen is too small")
+                    log.warn(" server: %sx%s vs client: %sx%s", avail_w, avail_h, self.cx(root_w), self.cy(root_h))
+                    log.warn(" you may see strange behavior,")
+                    log.warn(" please see https://github.com/Xpra-org/xpra/blob/master/docs/Usage/Xdummy.md")
         #now that we have the server's screen info, allow scale changes:
         self.scale_change_embargo = 0
         self.set_max_packet_size()
@@ -289,7 +300,7 @@ class DisplayClient(StubClientMixin):
             server_w, server_h = self.server_actual_desktop_size
             maxw = max(root_w, server_w)
             maxh = max(root_h, server_h)
-        except ValueError:
+        except (TypeError, ValueError):
             pass
         if maxw<=0 or maxh<=0 or maxw>=32768 or maxh>=32768:
             message = "invalid maximum desktop size: %ix%i" % (maxw, maxh)
@@ -316,6 +327,9 @@ class DisplayClient(StubClientMixin):
     def get_display_icc_info(self) -> dict:
         return get_display_icc_info()
 
+    def get_monitors_info(self):
+        return {}
+
     def _process_show_desktop(self, packet):
         show = packet[1]
         log("calling %s(%s)", show_desktop, show)
@@ -482,8 +496,12 @@ class DisplayClient(StubClientMixin):
         ydpi = self.cy(ydpi)
         log("get_screen_settings() scaled: xdpi=%s, ydpi=%s", xdpi, ydpi)
         vrefresh = self.get_vrefresh()
-        log("get_screen_settings() vrefresh=%s", vrefresh)
-        return (root_w, root_h, sss, ndesktops, desktop_names, u_root_w, u_root_h, xdpi, ydpi, vrefresh)
+        rrate = -1
+        if vrefresh>0:
+            rrate = (get_refresh_rate_for_value(self.refresh_rate, vrefresh) or 0)//1000
+        log("get_screen_settings() vrefresh=%s (from %s)", rrate, vrefresh)
+        monitors = self.get_monitors_info()
+        return (root_w, root_h, sss, ndesktops, desktop_names, u_root_w, u_root_h, xdpi, ydpi, rrate, monitors)
 
     def update_screen_size(self):
         self.screen_size_change_timer = None
@@ -511,15 +529,15 @@ class DisplayClient(StubClientMixin):
 
     def scaleup(self):
         scaling = max(self.xscale, self.yscale)
-        options = [v for v in SCALING_OPTIONS if r4cmp(v, 10)>r4cmp(scaling, 10)]
-        scalinglog("scaleup() options>%s : %s", r4cmp(scaling, 1000)/1000.0, options)
+        options = scaleup_value(scaling)
+        scalinglog("scaleup() options=%s", options)
         if options:
             self._scaleto(min(options))
 
     def scaledown(self):
-        scaling = max(self.xscale, self.yscale)
-        options = [v for v in SCALING_OPTIONS if r4cmp(v, 10)<r4cmp(scaling, 10)]
-        scalinglog("scaledown() options<%s : %s", r4cmp(scaling, 1000)/1000.0, options)
+        scaling = min(self.xscale, self.yscale)
+        options = scaledown_value(scaling)
+        scalinglog("scaledown() options=%s", options)
         if options:
             self._scaleto(max(options))
 
@@ -600,6 +618,11 @@ class DisplayClient(StubClientMixin):
             scalinglog.info("setting scaling to %i%% x %i%%:", round(100*self.xscale), round(100*self.yscale))
         self.update_screen_size()
         #re-initialize all the windows with their new size
+        for win in self._id_to_window.values():
+            if hasattr(win, "_xscale"):
+                win._xscale *= xchange
+            if hasattr(win, "_yscale"):
+                win._yscale *= ychange
         def new_size_fn(w, h):
             minx, miny = 16384, 16384
             if self.max_window_size!=(0, 0):
