@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2019-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2019-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -7,20 +7,20 @@ import os
 import curses
 import signal
 import traceback
+from time import monotonic
 from subprocess import Popen, PIPE, DEVNULL
 from datetime import datetime, timedelta
 
-from xpra.version_util import caps_to_version
-from xpra.util import typedict, std, envint, csv, engs, repr_ellipsized
+from xpra.version_util import caps_to_version, full_version_str
+from xpra.util import u, noerr, typedict, std, envint, csv
 from xpra.os_util import (
     platform_name, get_machine_id,
-    bytestostr, monotonic_time,
+    bytestostr, strtobytes,
     POSIX, SIGNAMES,
     )
 from xpra.exit_codes import EXIT_STR
 from xpra.make_thread import start_thread
-from xpra.client.gobject_client_base import MonitorXpraClient
-from xpra.gtk_common.gobject_compat import register_os_signals
+from xpra.client.gobject_client_base import InfoTimerClient
 from xpra.platform.dotxpra import DotXpra
 from xpra.platform.paths import get_nodock_command
 from xpra.simple_stats import std_unit
@@ -46,8 +46,7 @@ SIGNAL_KEYS = {
 
 
 def get_title():
-    from xpra.version_util import full_version_str
-    return "Xpra top %s" % full_version_str()
+    return f"Xpra top {full_version_str()}"
 
 def curses_init():
     stdscr = curses.initscr()
@@ -138,14 +137,14 @@ class TopClient:
     def update_loop(self):
         while self.exit_code is None:
             self.update_screen()
-            elapsed = int(1000*monotonic_time()-self.last_getch)
+            elapsed = int(1000*monotonic()-self.last_getch)
             delay = max(100, min(1000, 1000-elapsed))//100
             curses.halfdelay(delay)
             try:
                 v = self.stdscr.getch()
             except Exception:
                 v = -1
-            self.last_getch = int(1000*monotonic_time())
+            self.last_getch = int(1000*monotonic())
             if v in EXIT_KEYS:
                 self.exit_code = 0
             if v in SIGNAL_KEYS:
@@ -172,19 +171,20 @@ class TopClient:
             env["XPRA_CONNECT_TIMEOUT"] = "3"
             proc = self.do_run_subcommand("top", env=env)
             if not proc:
-                self.message = monotonic_time(), "failed to execute subprocess", curses.color_pair(RED)
+                self.message = monotonic(), "failed to execute subprocess", curses.color_pair(RED)
                 return
             exit_code = proc.wait()
             txt = "top subprocess terminated"
             attr = 0
             if exit_code!=0:
                 attr = curses.color_pair(RED)
-                txt += " with error code %i" % exit_code
+                txt += f" with error code {exit_code}"
                 if exit_code in EXIT_STR:
-                    txt += " (%s)" % EXIT_STR.get(exit_code, "").replace("_", " ")
+                    estr = EXIT_STR.get(exit_code, "").replace("_", " ")
+                    txt += f" ({estr})"
                 elif (exit_code-128) in SIGNAMES:   #pylint: disable=superfluous-parens
-                    txt += " (%s)" % SIGNAMES[exit_code-128]
-            self.message = monotonic_time(), txt, attr
+                    txt += f" ({SIGNAMES[exit_code-128]})"
+            self.message = monotonic(), txt, attr
         finally:
             self.setup()
 
@@ -226,7 +226,7 @@ class TopClient:
             for sessions in sd.values():
                 for state, display, path in sessions:
                     displays.setdefault(display, []).append((state, path))
-            self.stdscr.addstr(hpos, 0, "found %i display%s:" % (len(displays), engs(displays)))
+            self.stdscr.addstr(hpos, 0, f"found {len(displays)} displays:")
             self.position = min(len(displays), self.position)
             self.selected_session = None
             hpos += 1
@@ -234,7 +234,7 @@ class TopClient:
                 return
             if self.message:
                 ts, txt, attr = self.message
-                if monotonic_time()-ts<10:
+                if monotonic()-ts<10:
                     self.stdscr.addstr(hpos, 0, txt, attr)
                     hpos += 1
                     if height<=hpos:
@@ -271,6 +271,7 @@ class TopClient:
         for state, path in state_paths:
             sinfo = "%50s : %s" % (path, state)
             if POSIX:
+                # pylint: disable=import-outside-toplevel
                 from pwd import getpwuid
                 from grp import getgrgid
                 try:
@@ -280,7 +281,7 @@ class TopClient:
                     #if stat.st_gid!=os.getgid():
                     sinfo += "  gid=%s" % getgrgid(stat.st_gid).gr_name
                 except Exception as e:
-                    sinfo += "(stat error: %s)" % e
+                    sinfo += f"(stat error: {e})"
             info.append(sinfo)
             if state==DotXpra.LIVE:
                 valid_path = path
@@ -291,10 +292,10 @@ class TopClient:
             stype = d.get("session-type")
             error = d.get("error")
             if error:
-                info[0] = "%s  %s" % (display, error)
+                info[0] = f"{display}  {error}"
             else:
-                info[0] = "%s  %s" % (display, name)
-                info.insert(1, "uuid=%s, type=%s" % (uuid, stype))
+                info[0] = f"{display}  {name}"
+                info.insert(1, f"uuid={uuid}, type={stype}")
             machine_id = d.get("machine-id")
             if machine_id is None or machine_id==get_machine_id():
                 try:
@@ -305,12 +306,12 @@ class TopClient:
                     try:
                         process = self.psprocess.get(pid)
                         if not process:
-                            import psutil
+                            import psutil  #@UnresolvedImport pylint: disable=import-outside-toplevel
                             process = psutil.Process(pid)
                             self.psprocess[pid] = process
                         else:
                             cpu = process.cpu_percent()
-                            info[0] += ", %i%% CPU" % (cpu)
+                            info[0] += ", %3i%% CPU" % (cpu)
                     except Exception:
                         pass
         return info
@@ -318,7 +319,7 @@ class TopClient:
     def get_display_id_info(self, path):
         d = {}
         try:
-            cmd = get_nodock_command()+["id", "socket://%s" % path]
+            cmd = get_nodock_command()+["id", f"socket://{path}"]
             proc = Popen(cmd, stdout=PIPE, stderr=PIPE)
             out, err = proc.communicate()
             for line in bytestostr(out or err).splitlines():
@@ -349,61 +350,43 @@ class TopClient:
         box(self.stdscr, x, y, w, h, ul, ur, ll, lr)
 
 
-class TopSessionClient(MonitorXpraClient):
+class TopSessionClient(InfoTimerClient):
 
     def __init__(self, *args):
         super().__init__(*args)
         self.log_file = None
         if CURSES_LOG:
             self.log_file = open(CURSES_LOG, "ab")
-        self.info_request_pending = False
-        self.server_last_info = typedict()
-        self.server_last_info_time = 0
-        self.info_timer = 0
         self.paused = False
         self.stdscr = None
+        self.modified = False
         self.psprocess = {}
-        self.setup()
         start_thread(self.input_thread, "input-thread", daemon=True)
 
     def client_type(self):
-        #overriden in subclasses!
         return "top"
 
     def server_connection_established(self, caps):
-        self.log("server_connection_established(%s)" % repr_ellipsized(caps))
-        self.log("traceback: %s" % (traceback.extract_stack(),))
-        r = super().server_connection_established(caps)
-        self.stdscr = curses_init()
+        self.log(f"server_connection_established({caps!r})")
+        self.log("traceback: " + str(traceback.extract_stack()))
+        self.setup()
         self.update_screen()
-        return r
+        return super().server_connection_established(caps)
 
     def setup(self):
-        self.stdscr = curses_init()
+        if self.stdscr is None:
+            self.stdscr = curses_init()
         try:
             curses.cbreak()
             curses.halfdelay(10)
         except Exception as e:
-            self.log("failed to configure curses: %s" % e)
-
-    def run(self):
-        register_os_signals(self.signal_handler, None)
-        v = super().run()
-        self.log("run()=%s" % v)
-        self.cleanup()
-        self.close_log()
-        return v
-
-    def signal_handler(self, signum, *args):
-        self.log("exit_code=%s" % self.exit_code)
-        self.log("signal_handler(%s, %s)" % (signum, args,))
-        self.quit(128+signum)
-        self.log("exit_code=%s" % self.exit_code)
+            self.log(f"failed to configure curses: {e}")
 
     def cleanup(self):
-        self.cancel_info_timer()
-        MonitorXpraClient.cleanup(self)
+        super().cleanup()
         curses_clean(self.stdscr)
+        self.stdscr = None
+        self.close_log()
 
     def close_log(self):
         log_file = self.log_file
@@ -413,47 +396,65 @@ class TopSessionClient(MonitorXpraClient):
             log_file.close()
 
     def log(self, message):
-        if self.log_file:
+        lf = self.log_file
+        if lf:
             now = datetime.now()
-            self.log_file.write(("%s %s\n" % (now.strftime("%Y/%m/%d %H:%M:%S.%f"), message)).encode())
-            self.log_file.flush()
+            #we log from multiple threads,
+            #so the file may have been closed
+            #by the time we get here:
+            noerr(lf.write, (now.strftime("%Y/%m/%d %H:%M:%S.%f")+" "+message+"\n").encode())
+            noerr(lf.flush)
 
     def err(self, e):
-        if self.log_file:
-            self.log_file.write(b"%s\n" % e)
-            self.log_file.write(traceback.format_exc().encode())
+        lf = self.log_file
+        if lf:
+            noerr(lf.write, b"%s\n" % e)
+            noerr(lf.write(traceback.format_exc().encode()))
         else:
             curses_err(self.stdscr, e)
 
+
+    def dictwarn(self, msg, *args):
+        try:
+            self.log(msg % (args,))
+        except Exception as e:
+            self.log(f"error logging message: {e}")
+    def td(self, d):
+        d = typedict(d)
+        #override warning method so we don't corrupt the curses output
+        d.warn = self.dictwarn
+        return d
+
     def update_screen(self):
-        self.log("signal handlers=%s" % signal.getsignal(signal.SIGINT))
-        self.log("update_screen()")
-        if not self.paused:
-            self.stdscr.erase()
-            try:
-                self.do_update_screen()
-            except Exception as e:
-                self.err(e)
-            finally:
-                self.stdscr.refresh()
+        self.modified = True
 
     def input_thread(self):
+        self.log(f"input thread: signal handlers={signal.getsignal(signal.SIGINT)}")
         while self.exit_code is None:
+            if not self.paused and self.modified:
+                self.stdscr.erase()
+                try:
+                    self.do_update_screen()
+                except Exception as e:
+                    self.err(e)
+                finally:
+                    self.stdscr.refresh()
+                    self.modified = False
             try:
                 curses.halfdelay(10)
                 v = self.stdscr.getch()
             except Exception as e:
-                self.log("getch() %s" % e)
+                self.log(f"getch() {e}")
                 v = -1
-            self.log("getch()=%s" % v)
+            self.log(f"getch()={v}")
             if v==-1:
                 continue
             if v in EXIT_KEYS:
-                self.log("exit on key '%s'" % v)
+                self.log(f"exit on key {v!r}")
                 self.quit(0)
                 break
             if v in SIGNAL_KEYS:
-                self.log("exit on signal key '%s'" % v)
+                self.log(f"exit on signal key {v!r}")
                 self.quit(128+SIGNAL_KEYS[v])
                 break
             if v in PAUSE_KEYS:
@@ -484,15 +485,16 @@ class TopSessionClient(MonitorXpraClient):
             build = self.slidictget("server", "build")
             vstr = caps_to_version(build)
             mode = server_info.strget("mode", "server")
-            python_info = typedict(server_info.dictget("python", {}))
-            bits = python_info.intget("bits", 32)
-            server_str = "Xpra %s server version %s %i-bit" % (mode, vstr, bits)
+            python_info = self.td(server_info.dictget("python", {}))
+            bits = python_info.intget("bits", 0)
+            bitsstr = "" if bits==0 else f" {bits}-bit"
+            server_str = f"Xpra {mode} server version {vstr}{bitsstr}"
             proxy_info = self.slidictget("proxy")
             if proxy_info:
-                proxy_platform_info = typedict(proxy_info.dictget("platform", {}))
+                proxy_platform_info = self.td(proxy_info.dictget("platform", {}))
                 proxy_platform = proxy_platform_info.strget("")
                 proxy_release = proxy_platform_info.strget("release")
-                proxy_build_info = typedict(proxy_info.dictget("build", {}))
+                proxy_build_info = self.td(proxy_info.dictget("build", {}))
                 proxy_version = proxy_build_info.strget("version")
                 proxy_distro = proxy_info.strget("linux_distribution")
                 server_str += " via: %s proxy version %s" % (
@@ -530,18 +532,18 @@ class TopSessionClient(MonitorXpraClient):
                     try:
                         process = self.psprocess.get(server_pid)
                         if not process:
-                            import psutil
+                            import psutil  #@UnresolvedImport
                             process = psutil.Process(server_pid)
                             self.psprocess[server_pid] = process
                         else:
                             cpu = process.cpu_percent()
-                            rinfo += ", %i%% CPU" % (cpu)
+                            rinfo += ", %3i%% CPU" % (cpu)
                     except Exception:
                         pass
             cpuinfo = self.slidictget("cpuinfo")
             if cpuinfo:
                 rinfo += ", %s" % cpuinfo.strget("hz_actual")
-            elapsed = monotonic_time()-self.server_last_info_time
+            elapsed = monotonic()-self.server_last_info_time
             color = WHITE
             if self.server_last_info_time==0:
                 rinfo += " - no server data"
@@ -555,8 +557,13 @@ class TopSessionClient(MonitorXpraClient):
             dinfo = []
             server = self.slidictget("server")
             rws = server.intpair("root_window_size", None)
+            display_info = self.slidictget("display")
             if rws:
-                sinfo = "%ix%i display" % (rws[0], rws[1])
+                sinfo = "%ix%i" % (rws[0], rws[1])
+                depth = display_info.intget("depth")
+                if depth>0:
+                    sinfo += " %i-bit" % depth
+                sinfo += " display"
                 mds = server.intpair("max_desktop_size")
                 if mds:
                     sinfo += " (max %ix%i)" % (mds[0], mds[1])
@@ -565,7 +572,6 @@ class TopSessionClient(MonitorXpraClient):
             if cursor_info:
                 cx, cy = cursor_info.inttupleget("position", (0, 0))
                 dinfo.append("cursor at %ix%i" % (cx, cy))
-            display_info = self.slidictget("display")
             pid = display_info.intget("pid")
             if pid:
                 dinfo.append("pid %i" % pid)
@@ -578,35 +584,43 @@ class TopSessionClient(MonitorXpraClient):
                 addstr_main(5, 0, gl_info)
                 hpos += 1
 
+            #filter clients, only show GUI clients:
+            client_info = self.slidictget("client")
+            gui_clients = []
+            nclients = 0
+            while True:
+                ci = client_info.dictget(nclients)
+                if not ci:
+                    break
+                ci = self.td(ci)
+                session_id = ci.strget("session-id")
+                if session_id!=self.session_id and ci.boolget("windows", True) and ci.strget("type")!="top":
+                    gui_clients.append(nclients)
+                nclients += 1
+
+            ngui = 0
             if hpos<height-3:
                 hpos += 1
                 if nclients==0:
-                    addstr_main(hpos, 0, "no clients connected")
+                    clients_str = "no clients connected"
                 else:
-                    addstr_main(hpos, 0, "%i client%s connected:" % (nclients, engs(nclients)))
+                    ngui = len(gui_clients)
+                    clients_str = f"{nclients} clients connected, "
+                    if ngui==0:
+                        clients_str += "no gui clients"
+                    else:
+                        clients_str += f"{ngui} gui clients:"
+                addstr_main(hpos, 0, clients_str)
                 hpos += 1
-            client_info = self.slidictget("client")
-            client_no = 0
-            while True:
+            for client_index, client_no in enumerate(gui_clients):
                 ci = client_info.dictget(client_no)
-                if not ci:
-                    break
-                client_no +=1
-                ci = typedict(ci)
-                session_id = ci.strget("session-id")
-                if session_id:
-                    #don't show ourselves:
-                    if session_id==self.session_id:
-                        continue
-                elif not ci.boolget("windows", True):
-                    #for older servers, hide any client that doesn't display windows:
-                    continue
-                ci = self.get_client_info(ci)
+                assert ci
+                ci = self.get_client_info(self.td(ci))
                 l = len(ci)
                 if hpos+2+l>height:
                     if hpos<height:
-                        more = nclients-client_no
-                        addstr_box(hpos, 0, "%i client%s not shown" % (more, engs(more)), curses.A_BOLD)
+                        more = ngui-client_index
+                        addstr_box(hpos, 0, f"{more} clients not shown", curses.A_BOLD)
                     break
                 self.box(1, hpos, width-2, 2+l)
                 for i, info in enumerate(ci):
@@ -618,18 +632,19 @@ class TopSessionClient(MonitorXpraClient):
             windows = self.slidictget("windows")
             if hpos<height-3:
                 hpos += 1
-                addstr_main(hpos, 0, "%i window%s:" % (len(windows), engs(windows)))
+                addstr_main(hpos, 0, f"{len(windows)} windows:")
                 hpos += 1
             wins = tuple(windows.values())
             nwindows = len(wins)
             for win_no, win in enumerate(wins):
-                wi = self.get_window_info(typedict(win))
+                wi = self.get_window_info(self.td(win))
                 l = len(wi)
                 if hpos+2+l>height:
                     if hpos<height:
                         more = nwindows-win_no
-                        addstr_main(hpos, 0, "terminal window is too small: %i window%s not shown" % \
-                                           (more, engs(more)), curses.A_BOLD)
+                        addstr_main(hpos, 0,
+                                    f"terminal window is too small: {more} windows not shown",
+                                    curses.A_BOLD)
                     break
                 self.box(1, hpos, width-2, 2+l)
                 for i, info in enumerate(wi):
@@ -645,21 +660,21 @@ class TopSessionClient(MonitorXpraClient):
     def dictget(self, dictinstance, *parts):
         d = dictinstance
         for part in parts:
-            d = typedict(d.dictget(part, {}))
+            d = self.td(d.dictget(part, {}))
         return d
 
     def get_client_info(self, ci):
         #version info:
         ctype = ci.strget("type", "unknown")
-        title = "%s client version" % ctype
+        title = f"{ctype} client version "
         title += caps_to_version(ci)
         chost = ci.strget("hostname")
         conn_info = ""
         if chost:
-            conn_info = "connected from %s " % chost
+            conn_info = f"connected from {chost} "
         cinfo = ci.dictget("connection")
         if cinfo:
-            cinfo = typedict(cinfo)
+            cinfo = self.td(cinfo)
             conn_info += "using %s %s" % (cinfo.strget("type"), cinfo.strget("protocol-type"))
             conn_info += ", with %s and %s" % (cinfo.strget("encoder"), cinfo.strget("compressor"))
         gl_info = self.get_gl_info(ci.dictget("opengl"))
@@ -669,16 +684,13 @@ class TopSessionClient(MonitorXpraClient):
             audio_info.append(self._audio_info(ci, mode))
         audio_info.append(self._avsync_info(ci))
         #batch delay / latency:
-        b_info = typedict(ci.dictget("batch", {}))
-        bi_info = typedict(b_info.dictget("delay", {}))
+        b_info = self.td(ci.dictget("batch", {}))
+        bi_info = self.td(b_info.dictget("delay", {}))
         bcur = bi_info.intget("cur")
         bavg = bi_info.intget("avg")
-        batch_info = "batch delay: %i (%i)" %(
-            bcur,
-            bavg,
-            )
+        batch_info = f"batch delay: {bcur} ({bavg})"
         #client latency:
-        pl = self.slidictget("connection", "client", "ping_latency")
+        pl = self.dictget(ci, "connection", "client", "ping_latency")
         lcur = pl.intget("cur")
         lavg = pl.intget("avg")
         lmin = pl.intget("min")
@@ -687,23 +699,23 @@ class TopSessionClient(MonitorXpraClient):
             bl_color = YELLOW
         elif bcur>100 or (lcur>20 and lcur>3*lmin):
             bl_color = RED
-        batch_latency = batch_info.ljust(24)+"latency: %i (%i)" % (lcur, lavg)
+        batch_latency = batch_info.ljust(24)+f"latency: {lcur} ({lavg})"
         #speed / quality:
-        edict = typedict(ci.dictget("encoding") or {})
+        edict = self.td(ci.dictget("encoding") or {})
         qs_info = ""
         qs_color = GREEN
         if edict:
-            sinfo = typedict(edict.dictget("speed") or {})
+            sinfo = self.td(edict.dictget("speed") or {})
             if sinfo:
                 cur = sinfo.intget("cur")
                 avg = sinfo.intget("avg")
-                qs_info = "speed: %s%% (avg: %s%%)" % (cur, avg)
-            qinfo = typedict(edict.dictget("quality") or {})
+                qs_info = f"speed: {cur}%% (avg: f{avg}%%)"
+            qinfo = self.td(edict.dictget("quality") or {})
             if qinfo:
                 qs_info = qs_info.ljust(24)
                 cur = qinfo.intget("cur")
                 avg = qinfo.intget("avg")
-                qs_info += "quality: %s%% (avg: %s%%)" % (cur, avg)
+                qs_info += f"quality: {cur}%% (avg: {avg}%%)"
                 if avg<70:
                     qs_color = YELLOW
                 if avg<50:
@@ -720,14 +732,14 @@ class TopSessionClient(MonitorXpraClient):
     def _audio_info(self, ci, mode="speaker"):
         minfo = self.dictget(ci, "sound", mode)
         if not minfo:
-            return "%s off" % mode
-        minfo = typedict(minfo)
-        audio_info = "%s: %s" % (mode, minfo.strget("codec_description") or \
-                                 minfo.strget("codec") or \
-                                 minfo.strget("state", "unknown"))
+            return f"{mode} off"
+        minfo = self.td(minfo)
+        audio_info = f"{mode}: %s" % (minfo.strget("codec_description") or \
+                                      minfo.strget("codec") or \
+                                      minfo.strget("state", "unknown"))
         bitrate = minfo.intget("bitrate")
         if bitrate:
-            audio_info += " %sbps" % std_unit(bitrate)
+            audio_info += f" {std_unit(bitrate)}bps"
         return audio_info
 
     def _avsync_info(self, ci):
@@ -752,15 +764,17 @@ class TopSessionClient(MonitorXpraClient):
                 k = bytestostr(k)
                 if k=="gravity":
                     v = GRAVITY_STR.get(v, v)
-                return "%s=%s" % (k, str(v))
+                return f"{k}={v}"
             g_str += " - %s" % csv(sc_str(k, v) for k,v in sc.items())
         line1 = ""
         pid = wi.intget("pid", 0)
         if pid:
-            line1 = "pid %i: " % pid
-        title = wi.strget("title", "")
+            line1 = f"pid {pid}: "
+        title = wi.get("title", "")
+        if not isinstance(title, str):
+            title = u(strtobytes(title))
         if title:
-            line1 += ' "%s"' % title
+            line1 += f' "{title}"'
         attrs = [
             x for x in (
                 "above", "below", "bypass-compositor",
@@ -783,73 +797,28 @@ class TopSessionClient(MonitorXpraClient):
     def get_gl_info(self, gli):
         if not gli:
             return None
-        gli = typedict(gli)
+        gli = self.td(gli)
         def strget(key, sep="."):
             #fugly warning:
             #depending on where we get the gl info from,
             #the value might be a list of strings,
             #or a byte string...
-            v = gli.rawget(key)
+            v = gli.get(key)
             if isinstance(v, (tuple, list)):
                 return sep.join(bytestostr(x) for x in v)
             return bytestostr(v)
         if not gli.boolget("enabled", True):
-            return "OpenGL disabled %s" % gli.strget("message", "")
+            return "OpenGL disabled " + gli.strget("message", "")
         gl_info = "OpenGL %s enabled: %s" % (strget("opengl", "."), gli.strget("renderer") or gli.strget("vendor"))
         depth = gli.intget("depth")
         if depth not in (0, 24):
-            gl_info += ", %ibits" % depth
-        modes = gli.rawget("display_mode")
+            gl_info += f", {depth}bits"
+        modes = gli.get("display_mode")
         if modes:
-            gl_info += " - %s" % strget("display_mode", ", ")
+            gl_info += " - " + strget("display_mode", ", ")
         return gl_info
 
     def box(self, x, y, w, h):
         box(self.stdscr, x, y, w, h,
             ul=curses.ACS_ULCORNER, ur=curses.ACS_URCORNER,     #@UndefinedVariable
             ll=curses.ACS_LLCORNER, lr=curses.ACS_LRCORNER)     #@UndefinedVariable
-
-
-    def do_command(self, caps : typedict):
-        self.send_info_request()
-        self.timeout_add(REFRESH_RATE*1000, self.send_info_request)
-
-    def send_info_request(self):
-        self.log("send_info_request()")
-        categories = ()
-        if not self.info_request_pending:
-            self.info_request_pending = True
-            window_ids = ()    #no longer used or supported by servers
-            self.send("info-request", [self.uuid], window_ids, categories)
-        if not self.info_timer:
-            self.info_timer = self.timeout_add((REFRESH_RATE+2)*1000, self.info_timeout)
-        return True
-
-    def init_packet_handlers(self):
-        MonitorXpraClient.init_packet_handlers(self)
-        self.add_packet_handler("info-response", self._process_info_response, False)
-
-    def _process_server_event(self, packet):
-        self.log("server event: %s" % (packet,))
-        self.last_server_event = packet[1:]
-        self.update_screen()
-
-    def _process_info_response(self, packet):
-        self.log("info response: %s" % repr_ellipsized(packet))
-        self.cancel_info_timer()
-        self.info_request_pending = False
-        self.server_last_info = typedict(packet[1])
-        self.server_last_info_time = monotonic_time()
-        #log.info("server_last_info=%s", self.server_last_info)
-        self.update_screen()
-
-    def cancel_info_timer(self):
-        it = self.info_timer
-        if it:
-            self.info_timer = None
-            self.source_remove(it)
-
-    def info_timeout(self):
-        self.log("info timeout")
-        self.update_screen()
-        return True
