@@ -1,21 +1,26 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2011-2021 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2022 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2009, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+#pylint: disable=import-outside-toplevel
+
 from collections import namedtuple
 from threading import Lock
 
 from xpra.log import Logger
-from xpra.net.header import FLAGS_RENCODE, FLAGS_YAML, FLAGS_BENCODE, FLAGS_NOHEADER
+from xpra.net.header import (
+    FLAGS_RENCODE, FLAGS_RENCODEPLUS, FLAGS_YAML, FLAGS_BENCODE, FLAGS_NOHEADER,
+    pack_header,
+    )
 from xpra.util import envbool
 
 #all the encoders we know about, in best compatibility order:
-ALL_ENCODERS = ("rencode", "bencode", "yaml", "none")
+ALL_ENCODERS = ("rencode", "bencode", "yaml", "rencodeplus", "none")
 #order for performance:
-PERFORMANCE_ORDER = ("rencode", "bencode", "yaml")
+PERFORMANCE_ORDER = ("rencodeplus", "rencode", "bencode", "yaml")
 
 Encoding = namedtuple("Encoding", ["name", "flag", "version", "encode", "decode"])
 
@@ -23,12 +28,20 @@ ENCODERS = {}
 
 
 def init_rencode():
-    from rencode import dumps, loads, __version__
+    import rencode  # @UnresolvedImport
     rencode_lock = Lock()
+    rencode_dumps = rencode.dumps
     def do_rencode(v):
         with rencode_lock:
-            return dumps(v), FLAGS_RENCODE
-    return Encoding("rencode", FLAGS_RENCODE, __version__, do_rencode, loads)
+            return rencode_dumps(v), FLAGS_RENCODE
+    return Encoding("rencode", FLAGS_RENCODE, rencode.__version__, do_rencode, rencode.loads)
+
+def init_rencodeplus():
+    from xpra.net.rencodeplus import rencodeplus    #pylint: disable=no-name-in-module
+    rencodeplus_dumps = rencodeplus.dumps
+    def do_rencodeplus(v):
+        return rencodeplus_dumps(v), FLAGS_RENCODEPLUS
+    return Encoding("rencodeplus", FLAGS_RENCODEPLUS, rencodeplus.__version__, do_rencodeplus, rencodeplus.loads)
 
 def init_bencode():
     from xpra.net.bencode import bencode, bdecode, __version__
@@ -36,7 +49,8 @@ def init_bencode():
         return bencode(v), FLAGS_BENCODE
     def do_bdecode(data):
         packet, l = bdecode(data)
-        assert len(data)==l, "expected %i bytes, but got %i" % (l, len(data))
+        if len(data)!=l:
+            raise RuntimeError(f"expected {l} bytes, but got {len(data)}")
         return packet
     return Encoding("bencode", FLAGS_BENCODE, __version__, do_bencode, do_bdecode)
 
@@ -61,9 +75,13 @@ def init_none():
 
 def init_encoders(*names):
     for x in names:
-        if not envbool("XPRA_%s" % (x.upper()), True):
+        if x not in ALL_ENCODERS:
+            logger = Logger("network", "protocol")
+            logger.warn("Warning: invalid encoder '%s'", x)
             continue
-        fn = globals().get("init_%s" % x)
+        if not envbool("XPRA_"+x.upper(), True):
+            continue
+        fn = globals().get(f"init_{x}")
         try:
             e = fn()
             assert e
@@ -76,7 +94,7 @@ def init_all():
     init_encoders(*(list(ALL_ENCODERS)+["none"]))
 
 
-def get_packet_encoding_caps() -> dict:
+def get_packet_encoding_caps(full=True) -> dict:
     caps = {}
     for name in ALL_ENCODERS:
         d = caps.setdefault(name, {})
@@ -84,7 +102,8 @@ def get_packet_encoding_caps() -> dict:
         d[""] = e is not None
         if e is None:
             continue
-        d["version"] = e.version
+        if full:
+            d["version"] = e.version
     return caps
 
 def get_enabled_encoders(order=ALL_ENCODERS):
@@ -92,11 +111,15 @@ def get_enabled_encoders(order=ALL_ENCODERS):
 
 
 def get_encoder(e):
-    assert e in ALL_ENCODERS, "invalid encoder name: %s" % e
-    assert e in ENCODERS, "%s is not available" % e
+    if e not in ALL_ENCODERS:
+        raise ValueError(f"invalid encoder name {e!r}")
+    if e not in ENCODERS:
+        raise ValueError(f"{e!r} is not available")
     return ENCODERS[e].encode
 
 def get_packet_encoding_type(protocol_flags) -> str:
+    if protocol_flags & FLAGS_RENCODEPLUS:
+        return "rencodeplus"
     if protocol_flags & FLAGS_RENCODE:
         return "rencode"
     if protocol_flags & FLAGS_YAML:
@@ -109,7 +132,6 @@ class InvalidPacketEncodingException(Exception):
 
 
 def pack_one_packet(packet):
-    from xpra.net.header import pack_header
     ee = get_enabled_encoders()
     if ee:
         e = get_encoder(ee[0])
@@ -125,13 +147,14 @@ def decode(data, protocol_flags):
     e = ENCODERS.get(ptype)
     if e:
         return e.decode(data)
-    raise InvalidPacketEncodingException("%s decoder is not available" % ptype)
+    raise InvalidPacketEncodingException(f"{ptype!r} decoder is not available")
 
 
 def main(): # pragma: no cover
     from xpra.util import print_nested_dict
     from xpra.platform import program_context
     with program_context("Packet Encoding", "Packet Encoding Info"):
+        init_all()
         print_nested_dict(get_packet_encoding_caps())
 
 
