@@ -1,19 +1,27 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2013-2021 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2013-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import re
 import os
 import sys
-import signal
+import stat
 import uuid
-import time
+import signal
+import socket
 import struct
 import binascii
 import threading
+from time import monotonic, sleep
+from subprocess import PIPE, Popen
+
+# only mininal imports go at the top
+# so that this file can be included everywhere
+# without too many side-effects
+# pylint: disable=import-outside-toplevel
 
 SIGNAMES = {}
 for signame in (sig for sig in dir(signal) if sig.startswith("SIG") and not sig.startswith("SIG_")):
@@ -67,7 +75,7 @@ def get_frame_info(ignore_threads=()):
             #sanitize stack to prevent None values (which cause encoding errors with the bencoder)
             sanestack = []
             for e in stack:
-                sanestack.append(tuple([nn(x) for x in e]))
+                sanestack.append(tuple(nn(x) for x in e))
             info[i] = {
                 ""          : tident,
                 "stack"     : sanestack,
@@ -95,7 +103,7 @@ def get_sysconfig_info():
         "config-vars",
         "paths",
         ):
-        fn = "get_%s" % attr.replace("-", "_")
+        fn = "get_"+attr.replace("-", "_")
         getter = getattr(sysconfig, fn, None)
         if getter:
             try:
@@ -118,7 +126,7 @@ def bytestostr(x) -> str:
         return x.decode("latin1")
     return str(x)
 def hexstr(v) -> str:
-    return bytestostr(binascii.hexlify(strtobytes(v)))
+    return bytestostr(binascii.hexlify(memoryview_to_bytes(v)))
 
 
 util_logger = None
@@ -210,8 +218,8 @@ def platform_release(release):
         except Exception as e:
             get_util_logger().debug("platform_release(%s)", release, exc_info=True)
             get_util_logger().warn("Warning: failed to get release information")
-            get_util_logger().warn(" from '%s':", SYSTEMVERSION_PLIST)
-            get_util_logger().warn(" %s", e)
+            get_util_logger().warn(f" from {SYSTEMVERSION_PLIST}:")
+            get_util_logger().warn(f" {e}")
     return release
 
 
@@ -231,7 +239,7 @@ def platform_name(sys_platform=sys.platform, release=None) -> str:
             values += list(release)
         else:
             values.append(release)
-        return " ".join([str(x) for x in values if x])
+        return " ".join(str(x) for x in values if x and x!="unknown")
     for k,v in PLATFORMS.items():
         regexp = re.compile(k)
         if regexp.match(sys_platform):
@@ -279,9 +287,9 @@ def get_user_uuid() -> str:
         u.update(ustr.encode("utf-8"))
     uupdate(get_machine_id())
     if POSIX:
-        uupdate(u"/")
+        uupdate("/")
         uupdate(str(os.getuid()))
-        uupdate(u"/")
+        uupdate("/")
         uupdate(str(os.getgid()))
     uupdate(os.path.expanduser("~/"))
     return u.hexdigest()
@@ -305,7 +313,7 @@ def restore_script_env(env):
     if "_PYTHON_WRAPPER_VARS" in env:
         for v in env["_PYTHON_WRAPPER_VARS"].split():
             origv = "_" + v
-            if origv in env:
+            if env.get(origv):
                 env[v] = env[origv]
             elif v in env:
                 del env[v]
@@ -398,9 +406,15 @@ def is_AlmaLinux() -> bool:
 def is_RockyLinux() -> bool:
     return is_distribution_variant(b"Rocky Linux")
 
+def is_OracleLinux() -> bool:
+    return is_distribution_variant(b"Oracle Linux")
+
 def is_RedHat() -> bool:
     return is_distribution_variant(b"RedHat")
 
+def is_openSUSE() -> bool:
+    return is_distribution_variant(b"openSUSE")
+
 
 def is_arm() -> bool:
     import platform
@@ -413,10 +427,9 @@ def get_linux_distribution():
     if LINUX and not _linux_distribution:
         #linux_distribution is deprecated in Python 3.5 and it causes warnings,
         #so use our own code first:
-        import subprocess
         cmd = ["lsb_release", "-a"]
         try:
-            p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            p = Popen(cmd, stdout=PIPE, stderr=PIPE)
             out = p.communicate()[0]
             assert p.returncode==0 and out
         except Exception:
@@ -455,7 +468,7 @@ def get_loaded_kernel_modules(*modlist):
     loaded = []
     if LINUX and os.path.exists("/sys/module"):
         for mod in modlist:
-            if os.path.exists("/sys/module/%s" % mod):  # pragma: no cover
+            if os.path.exists(f"/sys/module/{mod}"):  # pragma: no cover
                 loaded.append(mod)
     return loaded
 
@@ -489,7 +502,7 @@ def do_get_generic_os_name() -> str:
 def filedata_nocrlf(filename) -> str:
     v = load_binary_file(filename)
     if v is None:
-        get_util_logger().error("failed to load '%s'", filename)
+        get_util_logger().error(f"failed to load {filename!r}")
         return None
     return v.strip(b"\n\r")
 
@@ -500,11 +513,19 @@ def load_binary_file(filename) -> bytes:
         with open(filename, "rb") as f:
             return f.read()
     except Exception as e:  # pragma: no cover
-        get_util_logger().debug("load_binary_file(%s)", filename, exc_info=True)
-        get_util_logger().warn("Warning: failed to load '%s':", filename)
-        get_util_logger().warn(" %s", e)
+        get_util_logger().debug(f"load_binary_file({filename})", exc_info=True)
+        get_util_logger().warn(f"Warning: failed to load {filename!r}")
+        get_util_logger().warn(f" {e}")
         return None
 
+def get_proc_cmdline(pid):
+    if pid and LINUX:
+        #try to find the command via /proc:
+        proc_cmd_line = os.path.join("/proc", f"{pid}", "cmdline")
+        if os.path.exists(proc_cmd_line):
+            return load_binary_file(proc_cmd_line).rstrip(b"\0").split(b"\0")
+    return None
+
 def parse_encoded_bin_data(data):
     if not data:
         return None
@@ -565,32 +586,6 @@ def livefds():
                 live.add(fd)
     return live
 
-def get_all_fds():
-    fd_dirs = ["/dev/fd", "/proc/self/fd"]
-    fds = []
-    for fd_dir in fd_dirs:
-        if os.path.exists(fd_dir):
-            for fd_str in os.listdir(fd_dir):
-                try:
-                    fd = int(fd_str)
-                    fds.append(fd)
-                except OSError:
-                    # This exception happens inevitably, because the fd used
-                    # by listdir() is already closed.
-                    pass
-            return fds
-    sys.stderr.write("Uh-oh, can't close fds, please port me to your system...\n")
-    return fds
-
-def close_all_fds(exceptions=()):
-    for fd in get_all_fds():
-        try:
-            if fd not in exceptions:
-                os.close(fd)
-        except OSError:
-            # This exception happens inevitably, because the fd used
-            # by listdir() is already closed.
-            pass
 
 def use_tty():
     from xpra.util import envbool
@@ -609,14 +604,13 @@ def shellsub(s, subs=None):
         for var,value in subs.items():
             try:
                 if isinstance(s, bytes):
-                    s = s.replace(("$%s" % var).encode(), str(value).encode())
+                    s = s.replace(f"${var}".encode(), str(value).encode())
                     s = s.replace(("${%s}" % var).encode(), str(value).encode())
                 else:
-                    s = s.replace("$%s" % var, str(value))
+                    s = s.replace(f"${var}", str(value))
                     s = s.replace("${%s}" % var, str(value))
             except (TypeError, ValueError):
-                raise Exception("failed to substitute '%s' with value '%s' (%s) in '%s'" % (
-                    var, value, type(value), s)) from None
+                raise Exception(f"failed to substitute {var!r} with value {value!r} ({type(value)}) in {s!r}") from None
     return s
 
 
@@ -659,20 +653,21 @@ def path_permission_info(filename, ftype=None):
         return []
     info = []
     try:
-        import stat
         stat_info = os.stat(filename)
         if not ftype:
             ftype = "file"
             if os.path.isdir(filename):
                 ftype = "directory"
-        info.append("permissions on %s %s: %s" % (ftype, filename, oct(stat.S_IMODE(stat_info.st_mode))))
+        operm = oct(stat.S_IMODE(stat_info.st_mode))
+        info.append(f"permissions on {ftype} {filename}: {operm}")
+        # pylint: disable=import-outside-toplevel
         import pwd
         import grp      #@UnresolvedImport
         user = pwd.getpwuid(stat_info.st_uid)[0]
         group = grp.getgrgid(stat_info.st_gid)[0]
-        info.append("ownership %s:%s" % (user, group))
+        info.append(f"ownership {user}:{group}")
     except Exception as e:
-        info.append("failed to query path information for '%s': %s" % (filename, e))
+        info.append(f"failed to query path information for {filename!r}: {e}")
     return info
 
 
@@ -728,22 +723,6 @@ class OSEnvContext:
         return "OSEnvContext"
 
 
-class FDChangeCaptureContext:
-    __slots__ = ("enter_fds", "exit_fds")
-    def __init__(self):
-        self.enter_fds = []
-        self.exit_fds = []
-    def __enter__(self):
-        self.enter_fds = get_all_fds()
-    def __exit__(self, *_args):
-        self.exit_fds = get_all_fds()
-    def __repr__(self):
-        return "FDChangeCaptureContext"
-    def get_new_fds(self):
-        return sorted(tuple(set(self.exit_fds)-set(self.enter_fds)))
-    def get_lost_fds(self):
-        return sorted(tuple(set(self.enter_fds)-set(self.exit_fds)))
-
 class DummyContextManager:
     __slots__ = ()
     def __enter__(self):
@@ -769,7 +748,7 @@ class nomodule_context:
             else:
                 sys.modules[self.module_name] = self.saved_module
     def __repr__(self):
-        return "nomodule_context(%s)" % self.module_name
+        return f"nomodule_context({self.module_name})"
 
 class umask_context:
     __slots__ = ("umask", "orig_umask")
@@ -780,7 +759,7 @@ class umask_context:
     def __exit__(self, *_args):
         os.umask(self.orig_umask)
     def __repr__(self):
-        return "umask_context(%s)" % self.umask
+        return f"umask_context({self.umask})"
 
 
 def disable_stdout_buffering():
@@ -817,7 +796,7 @@ def find_lib_ldconfig(libname):
             ldconfig = t
             break
     import subprocess
-    p = subprocess.Popen("%s -p" % ldconfig, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
+    p = subprocess.Popen(f"{ldconfig} -p", stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
     data = bytestostr(p.communicate()[0])
 
     libpath = re.search(pattern, data, re.MULTILINE)        #@UndefinedVariable
@@ -845,37 +824,42 @@ def find_lib(libname):
 
 
 def pollwait(process, timeout=5):
-    start = time.monotonic()
+    start = monotonic()
     v = None
-    while time.monotonic()-start<timeout:
+    while monotonic()-start<timeout:
         v = process.poll()
         if v is not None:
             break
-        time.sleep(0.1)
+        sleep(0.1)
     return v
 
+
+def find_in_PATH(command):
+    path = os.environ.get("PATH", None)
+    if not path:
+        return None
+    paths = path.split(os.pathsep)
+    for p in paths:
+        f = os.path.join(p, command)
+        if os.path.isfile(f):
+            return f
+    return None
+
 def which(command):
     try:
-        from distutils.spawn import find_executable
+        from shutil import which as find_executable
     except ImportError:
-        path = os.environ.get("PATH", None)
-        if not path:
-            return None
-        paths = path.split(os.pathsep)
-        for p in paths:
-            f = os.path.join(p, command)
-            if os.path.isfile(f):
-                return f
-        return None
-    else:
         try:
-            return find_executable(command)
-        except Exception:
-            get_util_logger().debug("find_executable(%s)", command, exc_info=True)
-            return None
+            from distutils.spawn import find_executable  # pylint: disable=deprecated-module
+        except ImportError:
+            find_executable = find_in_PATH
+    try:
+        return find_executable(command)
+    except Exception:
+        get_util_logger().debug(f"find_executable({command})", exc_info=True)
+        return None
 
 def get_status_output(*args, **kwargs):
-    from subprocess import PIPE, Popen
     kwargs.update({
         "stdout"    : PIPE,
         "stderr"    : PIPE,
@@ -884,7 +868,9 @@ def get_status_output(*args, **kwargs):
     try:
         p = Popen(*args, **kwargs)
     except Exception as e:
-        print("error running %s,%s: %s" % (args, kwargs, e))
+        from xpra.log import Logger
+        log = Logger("util")
+        log.error(f"Error running {args},{kwargs}: {e}")
         return -1, "", ""
     stdout, stderr = p.communicate()
     return p.returncode, stdout, stderr
@@ -915,7 +901,7 @@ def setuidgid(uid, gid):
         try:
             username = getpwuid(uid).pw_name
         except KeyError:
-            raise Exception("uid %i not found" % uid) from None
+            raise Exception(f"uid {uid} not found") from None
         #set the groups:
         if hasattr(os, "initgroups"):   # python >= 2.7
             os.initgroups(username, gid)
@@ -928,40 +914,55 @@ def setuidgid(uid, gid):
         if os.getgid()!=gid:
             os.setgid(gid)
     except OSError as e:
-        log.error("Error: cannot change gid to %i:", gid)
+        log.error(f"Error: cannot change gid to {gid}")
         if os.getgid()==0:
             #don't run as root!
             raise
-        log.error(" %s", e)
-        log.error(" continuing with gid=%i", os.getgid())
+        log.estr(e)
+        log.error(" continuing with gid={os.getgid()}")
     try:
         if os.getuid()!=uid:
             os.setuid(uid)
     except OSError as e:
-        log.error("Error: cannot change uid to %i:", uid)
+        log.error(f"Error: cannot change uid to {uid}")
         if os.getuid()==0:
             #don't run as root!
             raise
-        log.error(" %s", e)
-        log.error(" continuing with uid=%i", os.getuid())
-    log("new uid=%s, gid=%s", os.getuid(), os.getgid())
+        log.estr(e)
+        log.error(f" continuing with uid={os.getuid()}")
+    log(f"new uid={os.getuid()}, gid={os.getgid()}")
 
 def get_peercred(sock):
     if LINUX:
         SO_PEERCRED = 17
         log = get_util_logger()
         try:
-            import socket
             creds = sock.getsockopt(socket.SOL_SOCKET, SO_PEERCRED, struct.calcsize(b'3i'))
             pid, uid, gid = struct.unpack(b'3i',creds)
             log("peer: %s", (pid, uid, gid))
             return pid, uid, gid
         except IOError as  e:
             log("getsockopt", exc_info=True)
-            log.error("Error getting peer credentials: %s", e)
+            log.error(f"Error getting peer credentials: {e}")
             return None
     elif FREEBSD:
         log.warn("Warning: peercred is not yet implemented for FreeBSD")
         #use getpeereid
         #then pwd to get the gid?
     return None
+
+def is_socket(sockpath, check_uid=None):
+    try:
+        s = os.stat(sockpath)
+    except OSError as e:
+        get_util_logger().debug(f"is_socket({sockpath}) path cannot be accessed: {e}")
+        #socket cannot be accessed
+        return False
+    if not stat.S_ISSOCK(s.st_mode):
+        return False
+    if check_uid is not None:
+        if s.st_uid!=check_uid:
+            #socket uid does not match
+            get_util_logger().debug(f"is_socket({sockpath}) uid {s.st_uid} does not match {check_uid}")
+            return False
+    return True
