@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # This file is part of Xpra.
 # Copyright (c) 2016-2017 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
@@ -186,7 +186,7 @@ class NamedPipeConnection(Connection):
         return self.target
 
     def get_info(self) -> dict:
-        d = Connection.get_info(self)
+        d = super().get_info()
         d["type"] = "named-pipe"
         d["closed"] = self.pipe_handle is None
         return d
