@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2021 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -10,9 +10,9 @@ import struct
 
 from xpra.os_util import (
     bytestostr, hexstr, get_saved_env,
-    is_unity, is_gnome, is_kde, is_Ubuntu, is_Fedora, is_X11, is_Wayland, get_saved_env_var,
+    is_X11, is_Wayland, get_saved_env_var,
     )
-from xpra.util import u, envbool, envint, csv, ellipsizer
+from xpra.util import u, envbool, envint, csv, ellipsizer, first_time
 from xpra.log import Logger
 
 log = Logger("posix")
@@ -67,7 +67,7 @@ device_bell = None
 GTK_MENUS = envbool("XPRA_GTK_MENUS", False)
 RANDR_DPI = envbool("XPRA_RANDR_DPI", True)
 XSETTINGS_DPI = envbool("XPRA_XSETTINGS_DPI", True)
-USE_NATIVE_TRAY = envbool("XPRA_USE_NATIVE_TRAY", is_unity() or (is_Ubuntu() and is_gnome()) or (is_gnome() and not is_Fedora()) or is_kde())
+USE_NATIVE_TRAY = envbool("XPRA_USE_NATIVE_TRAY", True)
 XINPUT_WHEEL_DIV = envint("XPRA_XINPUT_WHEEL_DIV", 15)
 DBUS_SCREENSAVER = envbool("XPRA_DBUS_SCREENSAVER", False)
 
@@ -78,10 +78,6 @@ def gl_check():
     return None
 
 
-def get_native_system_tray_classes():
-    return []
-
-
 def get_wm_name():
     return do_get_wm_name(get_saved_env())
 
@@ -120,24 +116,30 @@ def get_clipboard_native_class():
         return "xpra.gtk_common.gtk_clipboard.GTK_Clipboard"
     return "xpra.x11.gtk_x11.clipboard.X11Clipboard"
 
+def get_native_system_tray_classes():
+    c = [_try_load_appindicator()]
+    traylog("get_native_system_tray_classes()=%s (USE_NATIVE_TRAY=%s)", c, USE_NATIVE_TRAY)
+    return c
+
 def get_native_tray_classes():
     #could restrict to only DEs that have a broken system tray like "GNOME Shell"?
-    c = []
-    if USE_NATIVE_TRAY:
-        try:
-            from xpra.platform.xposix.appindicator_tray import AppindicatorTray
-            c.append(AppindicatorTray)
-        except (ImportError, ValueError):
-            traylog("cannot load appindicator tray", exc_info=True)
-            traylog.warn("Warning: appindicator library not found")
-            traylog.warn(" you may want to install libappindicator")
-            traylog.warn(" to enable the system tray.")
-            if get_saved_env_var("XDG_CURRENT_DESKTOP")=="GNOME":
-                traylog.warn(" With gnome-shell, you may also need some extensions:")
-                traylog.warn(" 'top icons plus' and / or 'appindicator'")
+    c = [_try_load_appindicator()]
     traylog("get_native_tray_classes()=%s (USE_NATIVE_TRAY=%s)", c, USE_NATIVE_TRAY)
     return c
 
+def _try_load_appindicator():
+    try:
+        from xpra.platform.xposix.appindicator_tray import AppindicatorTray
+        return AppindicatorTray
+    except (ImportError, ValueError):
+        if first_time("no-appindicator"):
+             traylog("cannot load appindicator tray", exc_info=True)
+             traylog.warn("Warning: appindicator library not found")
+             traylog.warn(" you may want to install libappindicator")
+             if get_saved_env_var("XDG_CURRENT_DESKTOP", "").upper().find("GNOME")>=0:
+                 traylog.warn(" With gnome-shell, you may also need some extensions:")
+                 traylog.warn(" 'top icons plus' and / or 'appindicator'")
+    return None
 
 def get_native_notifier_classes():
     ncs = []
@@ -218,7 +220,10 @@ def _get_xsettings_dict():
 
 def _get_xsettings_dpi():
     if XSETTINGS_DPI and is_X11():
-        from xpra.x11.xsettings_prop import XSettingsTypeInteger
+        try:
+            from xpra.x11.xsettings_prop import XSettingsTypeInteger
+        except ImportError:
+            return -1
         d = _get_xsettings_dict()
         for k,div in {
             "Xft.dpi"         : 1,
@@ -420,7 +425,7 @@ def _get_xresources():
             from xpra.gtk_common.gtk_util import get_default_root_window
             root = get_default_root_window()
             value = prop_get(root, "RESOURCE_MANAGER", "latin1", ignore_errors=True)
-            log("RESOURCE_MANAGER=%s", value)
+            log(f"RESOURCE_MANAGER={value}")
             if value is None:
                 return None
             #parse the resources into a dict:
@@ -431,12 +436,12 @@ def _get_xresources():
                     continue
                 parts = option.split(":\t", 1)
                 if len(parts)!=2:
-                    log("skipped invalid option: '%s'", option)
+                    log(f"skipped invalid option: {option!r}")
                     continue
                 values[parts[0]] = parts[1]
             return values
         except Exception as e:
-            log("_get_xresources error: %s", e)
+            log(f"_get_xresources error: {e!r}")
     return None
 
 def get_cursor_size():
@@ -499,10 +504,30 @@ def system_bell(window, device, percent, _pitch, _duration, bell_class, bell_id,
         return False
 
 
+def pointer_grab(gdk_window):
+    if is_X11():
+        try:
+            from xpra.gtk_common.error import xsync
+            with xsync:
+                return X11WindowBindings().pointer_grab(gdk_window.get_xid())
+        except Exception:
+            log.error("Error: failed to grab pointer", exc_info=True)
+    return False
+
+def pointer_ungrab(_window):
+    if is_X11():
+        try:
+            from xpra.gtk_common.error import xsync
+            with xsync:
+                return X11WindowBindings().UngrabPointer()==0
+            return True
+        except Exception:
+            log.error("Error: failed to ungrab pointer", exc_info=True)
+    return False
+
+
 def _send_client_message(window, message_type, *values):
     try:
-        from xpra.x11.gtk_x11.gdk_display_source import init_gdk_display_source
-        init_gdk_display_source()
         from xpra.x11.bindings.window_bindings import constants #@UnresolvedImport
         X11Window = X11WindowBindings()
         root_xid = X11Window.getDefaultRootWindow()
@@ -837,6 +862,15 @@ class ClientExtras:
             dbuslog.error(" %s", e)
             return
 
+        try:
+            import dbus
+            assert dbus
+        except ImportError as e:
+            dbuslog("setup_dbus_signals()", exc_info=True)
+            dbuslog.warn("Warning: cannot setup dbus signals")
+            dbuslog.warn(f" {e}")
+            return
+
         try:
             bus = init_system_bus()
             self.system_bus = bus
@@ -947,9 +981,9 @@ class ClientExtras:
             return False
         try:
             from xpra.gtk_common.error import xsync, XError
-            assert X11WindowBindings, "no X11 window bindings"
-            assert X11XI2Bindings, "no XI2 window bindings"
+            assert X11WindowBindings(), "no X11 window bindings"
             XI2 = X11XI2Bindings()
+            assert XI2, "no XI2 window bindings"
             #this may fail when windows are being destroyed,
             #ie: when another client disconnects because we are stealing the session
             try:
@@ -983,10 +1017,12 @@ class ClientExtras:
 
 
     def _get_xsettings(self):
-        try:
-            return self._xsettings_watcher.get_settings()
-        except Exception:
-            log.error("failed to get XSETTINGS", exc_info=True)
+        xw = self._xsettings_watcher
+        if xw:
+            try:
+                return xw.get_settings()
+            except Exception:
+                log.error("failed to get XSETTINGS", exc_info=True)
         return None
 
     def _handle_xsettings_changed(self, *_args):
