@@ -8,7 +8,7 @@ import os
 import sys
 import socket
 
-from xpra.util import envint, obsc, typedict
+from xpra.util import envint, obsc, typedict, stderr_write
 from xpra.os_util import bytestostr
 from xpra.server.auth.sys_auth_base import SysAuthenticatorBase, log, parse_uid, parse_gid
 from xpra.log import is_debug_enabled, enable_debug_for
@@ -18,6 +18,7 @@ LDAP_PROTOCOL_VERSION = envint("XPRA_LDAP_PROTOCOL_VERSION", 3)
 LDAP_TRACE_LEVEL = envint("XPRA_LDAP_TRACE_LEVEL")
 LDAP_CACERTFILE = os.environ.get("XPRA_LDAP_CACERTFILE")
 LDAP_ENCODING = os.environ.get("XPRA_LDAP_ENCODING", "utf-8")
+LDAP_USERNAME_FORMAT = os.environ.get("XPRA_LDAP_USERNAME_FORMAT", "cn=%username, o=%domain")
 
 
 class Authenticator(SysAuthenticatorBase):
@@ -35,7 +36,7 @@ class Authenticator(SysAuthenticatorBase):
         else:
             default_port = 389
         self.port = int(kwargs.pop("port", default_port))
-        self.username_format = kwargs.pop("username_format", "cn=%username, o=%domain")
+        self.username_format = kwargs.pop("username_format", LDAP_USERNAME_FORMAT)
         #self.username_format = kwargs.pop("username_format", "%username@%domain")
         super().__init__(**kwargs)
         log("ldap auth: host=%s, port=%i, tls=%s, username_format=%s, cacert=%s, encoding=%s",
@@ -132,8 +133,8 @@ def main(argv):
                 enable_debug_for("auth")
                 argv.remove(x)
         if len(argv) not in (3,4,5,6,7):
-            sys.stderr.write("%s invalid arguments\n" % argv[0])
-            sys.stderr.write("usage: %s username password [host] [port] [tls] [username_format]\n" % argv[0])
+            stderr_write("%s invalid arguments\n" % argv[0])
+            stderr_write("usage: %s username password [host] [port] [tls] [username_format]\n" % argv[0])
             return 1
         username = argv[1]
         password = argv[2]
