@@ -7,6 +7,7 @@
 import os
 import sys
 import time
+from time import monotonic
 from multiprocessing import Queue as MQueue, freeze_support #@UnresolvedImport
 from gi.repository import GLib
 
@@ -17,13 +18,13 @@ from xpra.util import (
 from xpra.os_util import (
     get_username_for_uid, get_groups, get_home_for_uid, bytestostr,
     getuid, getgid, WIN32, POSIX, OSX,
-    monotonic_time, umask_context, get_group_id,
+    umask_context, get_group_id,
     )
 from xpra.net.socket_util import SOCKET_DIR_MODE, SOCKET_DIR_GROUP
 from xpra.server.server_core import ServerCore
 from xpra.server.control_command import ArgsControlCommand, ControlError
 from xpra.child_reaper import getChildReaper
-from xpra.scripts.parsing import parse_bool
+from xpra.scripts.parsing import parse_bool, MODE_ALIAS
 from xpra.scripts.config import make_defaults_struct, PROXY_START_OVERRIDABLE_OPTIONS, OPTION_TYPES
 from xpra.scripts.main import parse_display_name, connect_to, start_server_subprocess
 from xpra.make_thread import start_thread
@@ -71,7 +72,7 @@ class ProxyServer(ServerCore):
 
     def __init__(self):
         log("ProxyServer.__init__()")
-        ServerCore.__init__(self)
+        super().__init__()
         self._max_connections = MAX_CONCURRENT_CONNECTIONS
         self._start_sessions = False
         self.session_type = "proxy"
@@ -147,7 +148,7 @@ class ProxyServer(ServerCore):
                 log.error(" %s", e)
 
     def init_control_commands(self):
-        ServerCore.init_control_commands(self)
+        super().init_control_commands()
         self.control_commands["stop"] = ArgsControlCommand("stop", "stops the proxy instance on the given display",
                                                            self.handle_stop_command, min_args=1, max_args=1)
 
@@ -164,7 +165,7 @@ class ProxyServer(ServerCore):
 
 
     def init_packet_handlers(self):
-        ServerCore.init_packet_handlers(self)
+        super().init_packet_handlers()
         #add shutdown handler
         self._default_packet_handlers["shutdown-server"] = self._process_proxy_shutdown_server
 
@@ -181,7 +182,10 @@ class ProxyServer(ServerCore):
         return "proxy"
 
     def init_aliases(self):
-        pass
+        """
+        It is a lot less confusing if proxy servers don't use packet aliases at all.
+        So we override the aliases initialization and skip it.
+        """
 
     def do_run(self):
         self.main_loop = GLib.MainLoop()
@@ -235,10 +239,10 @@ class ProxyServer(ServerCore):
     def cleanup(self):
         self.stop_all_proxies()
         super().cleanup()
-        start = monotonic_time()
+        start = monotonic()
         live = True
         log("cleanup() proxy instances: %s", self.instances)
-        while monotonic_time()-start<PROXY_CLEANUP_GRACE_PERIOD and live:
+        while monotonic()-start<PROXY_CLEANUP_GRACE_PERIOD and live:
             live = tuple(x for x in tuple(self.instances.keys()) if x.is_alive())
             if live:
                 log("cleanup() still %i proxies alive: %s", len(live), live)
@@ -266,8 +270,8 @@ class ProxyServer(ServerCore):
         if not protocol.is_closed():
             self.send_disconnect(protocol, LOGIN_TIMEOUT)
 
-    def hello_oked(self, proto, packet, c, auth_caps):
-        if super().hello_oked(proto, packet, c, auth_caps):
+    def hello_oked(self, proto, c, auth_caps):
+        if super().hello_oked(proto, c, auth_caps):
             #already handled in superclass
             return
         self.accept_client(proto, c)
@@ -412,7 +416,7 @@ class ProxyServer(ServerCore):
                 return
             if display:
                 if display not in displays:
-                    if (":%s" % display) in displays:
+                    if ":%s" % display in displays:
                         display = ":%s" % display
                     else:
                         disconnect(SESSION_NOT_FOUND, "display '%s' not found" % display)
@@ -474,11 +478,13 @@ class ProxyServer(ServerCore):
             return
         log("server connection=%s", server_conn)
 
-        cipher = None
+        cipher = cipher_mode = None
         encryption_key = None
         if auth_caps:
             cipher = auth_caps.get("cipher")
             if cipher:
+                from xpra.net.crypto import DEFAULT_MODE
+                cipher_mode = auth_caps.get("cipher.mode", DEFAULT_MODE)
                 encryption_key = self.get_encryption_key(client_proto.authenticators, client_proto.keyfile)
 
         use_thread = PROXY_INSTANCE_THREADED
@@ -497,7 +503,7 @@ class ProxyServer(ServerCore):
             from xpra.server.proxy.proxy_instance_thread import ProxyInstanceThread
             pit = ProxyInstanceThread(session_options, self.video_encoders, self.pings,
                                       client_proto, server_conn,
-                                      disp_desc, cipher, encryption_key, c)
+                                      disp_desc, cipher, cipher_mode, encryption_key, c)
             pit.stopped = self.reap
             pit.run()
             self.instances[pit] = (False, display, None)
@@ -531,7 +537,7 @@ class ProxyServer(ServerCore):
                 process = ProxyInstanceProcess(uid, gid, env_options, session_options, self._socket_dir,
                                                self.video_encoders, self.pings,
                                                client_conn, disp_desc, client_state,
-                                               cipher, encryption_key, server_conn, c, message_queue)
+                                               cipher, cipher_mode, encryption_key, server_conn, c, message_queue)
                 log("starting %s from pid=%s", process, os.getpid())
                 self.instances[process] = (True, display, message_queue)
                 process.start()
@@ -560,9 +566,11 @@ class ProxyServer(ServerCore):
     def start_new_session(self, username, _password, uid, gid, new_session_dict=None, displays=()):
         log("start_new_session%s", (username, "..", uid, gid, new_session_dict, displays))
         sns = typedict(new_session_dict or {})
-        mode = sns.get("mode", "start")
-        assert mode in ("start", "start-desktop", "shadow"), "invalid start-new-session mode '%s'" % mode
-        display = sns.get("display")
+        mode = sns.strget("mode", "start")
+        mode = MODE_ALIAS.get(mode, mode)
+        if mode not in ("seamless", "desktop", "shadow", "monitor", "expand"):
+            raise ValueError(f"invalid start-new-session mode {mode!r}")
+        display = sns.strget("display")
         if display in displays:
             raise Exception("display %s is already active!" % display)
         log("starting new server subprocess: mode=%s, display=%s", mode, display)
@@ -653,7 +661,7 @@ class ProxyServer(ServerCore):
     def get_info(self, proto, *_args):
         authenticated = proto and proto.authenticators
         if not authenticated:
-            info = super().get_server_info()
+            info = self.get_minimal_server_info()
         else:
             #only show more info if we have authenticated
             #as the user running the proxy server process:
