@@ -117,7 +117,7 @@ class GTKShadowServerBase(ShadowServerBase, GTKServerBase):
                     #capture doesn't have any screen updates,
                     #so we can skip calling damage
                     #(this shortcut is only used with nvfbc)
-                    return False
+                    return True
             except TransientCodecException as e:
                 log("refresh()", exc_info=True)
                 log.warn("Warning: transient codec exception:")
@@ -370,6 +370,7 @@ class GTKShadowServerBase(ShadowServerBase, GTKServerBase):
             self.tray_widget = self.make_tray_widget()
             self.set_tray_icon(self.tray_icon  or "server-notconnected")
         except ImportError as e:
+            traylog("setup_tray()", exc_info=True)
             traylog.warn("Warning: failed to load systemtray:")
             traylog.warn(" %s", e)
         except Exception as e:
