@@ -1,21 +1,21 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2022 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
-from time import sleep
+from time import sleep, monotonic
 from threading import Event
 from collections import deque
 from queue import Queue
 
 from xpra.make_thread import start_thread
-from xpra.os_util import monotonic_time
+from xpra.common import FULL_INFO
 from xpra.util import notypedict, envbool, envint, typedict, AtomicInteger
-from xpra.net.compression import compressed_wrapper, Compressed
+from xpra.net.compression import compressed_wrapper
 from xpra.server.source.source_stats import GlobalPerformanceStatistics
 from xpra.server.source.stub_source_mixin import StubSourceMixin
 from xpra.log import Logger
@@ -34,25 +34,24 @@ YIELD = envbool("XPRA_YIELD", False)
 counter = AtomicInteger()
 
 
-"""
-This class mediates between the server class (which only knows about actual window objects and display server events)
-and the client specific WindowSource instances (which only know about window ids
-and manage window pixel compression).
-It sends messages to the client via its 'protocol' instance (the network connection),
-directly for a number of cases (cursor, sound, notifications, etc)
-or on behalf of the window sources for pixel data.
-
-Strategy: if we have 'ordinary_packets' to send, send those.
-When we don't, then send packets from the 'packet_queue'. (compressed pixels or clipboard data)
-See 'next_packet'.
-
-The UI thread calls damage(), which goes into WindowSource and eventually (batching may be involved)
-adds the damage pixels ready for processing to the encode_work_queue,
-items are picked off by the separate 'encode' thread (see 'encode_loop')
-and added to the damage_packet_queue.
-"""
-
 class ClientConnection(StubSourceMixin):
+    """
+    This class mediates between the server class (which only knows about actual window objects and display server events)
+    and the client specific WindowSource instances (which only know about window ids
+    and manage window pixel compression).
+    It sends messages to the client via its 'protocol' instance (the network connection),
+    directly for a number of cases (cursor, sound, notifications, etc)
+    or on behalf of the window sources for pixel data.
+
+    Strategy: if we have 'ordinary_packets' to send, send those.
+    When we don't, then send packets from the 'packet_queue'. (compressed pixels or clipboard data)
+    See 'next_packet'.
+
+    The UI thread calls damage(), which goes into WindowSource and eventually (batching may be involved)
+    adds the damage pixels ready for processing to the encode_work_queue,
+    items are picked off by the separate 'encode' thread (see 'encode_loop')
+    and added to the damage_packet_queue.
+    """
 
     def __init__(self, protocol, disconnect_cb, session_name,
                  setting_changed,
@@ -61,7 +60,7 @@ class ClientConnection(StubSourceMixin):
         global counter
         self.counter = counter.increase()
         self.protocol = protocol
-        self.connection_time = monotonic_time()
+        self.connection_time = monotonic()
         self.close_event = Event()
         self.disconnect = disconnect_cb
         self.session_name = session_name
@@ -102,6 +101,7 @@ class ClientConnection(StubSourceMixin):
         self.share = False
         self.lock = False
         self.control_commands = ()
+        self.xdg_menu = True
         self.xdg_menu_update = False
         self.bandwidth_limit = self.server_bandwidth_limit
         self.soft_bandwidth_limit = self.bandwidth_limit
@@ -110,15 +110,11 @@ class ClientConnection(StubSourceMixin):
         self.client_connection_data = {}
         self.adapter_type = ""
         self.jitter = 0
+        self.ssh_auth_sock = ""
         #what we send back in hello packet:
         self.ui_client = True
-        self.wants_aliases = True
-        self.wants_encodings = True
-        self.wants_versions = True
-        self.wants_features = True
-        self.wants_display = True
-        self.wants_events = False
-        self.wants_default_cursor = False
+        #default 'wants' is not including "events" or "default_cursor":
+        self.wants = ["aliases", "encodings", "versions", "features", "display"]
         #these statistics are shared by all WindowSource instances:
         self.statistics = GlobalPerformanceStatistics()
 
@@ -142,15 +138,11 @@ class ClientConnection(StubSourceMixin):
             notification_mixin.NotificationMixin.may_notify(self, *args, **kwargs)
 
 
-    def compressed_wrapper(self, datatype, data, min_saving=128):
-        if self.zlib or self.lz4 or self.lzo:
-            cw = compressed_wrapper(datatype, data, zlib=self.zlib, lz4=self.lz4, lzo=self.lzo, can_inline=False)
-            if len(cw)+min_saving<=len(data):
-                #the compressed version is smaller, use it:
-                return cw
-            #skip compressed version: fall through
-        #we can't compress, so at least avoid warnings in the protocol layer:
-        return Compressed(datatype, data, can_inline=True)
+    def compressed_wrapper(self, datatype, data, **kwargs):
+        #set compression flags based on self.zlib and self.lz4:
+        kw = dict((k, getattr(self, k, False)) for k in ("zlib", "lz4"))
+        kw.update(kwargs)
+        return compressed_wrapper(datatype, data, can_inline=False, **kw)
 
 
     def update_bandwidth_limits(self):
@@ -199,10 +191,11 @@ class ClientConnection(StubSourceMixin):
 
     def parse_client_caps(self, c : typedict):
         #general features:
-        self.info_namespace = c.boolget("info-namespace")
+        self.info_namespace = c.boolget("info-namespace", True)
         self.share = c.boolget("share")
         self.lock = c.boolget("lock")
         self.control_commands = c.strtupleget("control_commands")
+        self.xdg_menu = c.boolget("xdg-menu", True)
         self.xdg_menu_update = c.boolget("xdg-menu-update")
         bandwidth_limit = c.intget("bandwidth-limit", 0)
         server_bandwidth_limit = self.server_bandwidth_limit
@@ -217,6 +210,7 @@ class ClientConnection(StubSourceMixin):
         self.jitter = ccd.intget("jitter", 0)
         bandwidthlog("server bandwidth-limit=%s, client bandwidth-limit=%s, value=%s, detection=%s",
                      server_bandwidth_limit, bandwidth_limit, self.bandwidth_limit, self.bandwidth_detection)
+        self.ssh_auth_sock = c.strget("ssh-auth-sock", "")
 
         if getattr(self, "mmap_size", 0)>0:
             log("mmap enabled, ignoring bandwidth-limit")
@@ -265,7 +259,7 @@ class ClientConnection(StubSourceMixin):
             This is used by WindowSource to queue damage processing to be done in the 'encode' thread.
             The 'encode_and_send_cb' will then add the resulting packet to the 'packet_queue' via 'queue_packet'.
         """
-        self.statistics.compression_work_qsizes.append((monotonic_time(), self.encode_queue_size()))
+        self.statistics.compression_work_qsizes.append((monotonic(), self.encode_queue_size()))
         self.queue_encode(fn_and_args)
 
     def queue_packet(self, packet, wid=0, pixels=0,
@@ -274,7 +268,7 @@ class ClientConnection(StubSourceMixin):
             Add a new 'draw' packet to the 'packet_queue'.
             Note: this code runs in the non-ui thread
         """
-        now = monotonic_time()
+        now = monotonic()
         self.statistics.packet_qsizes.append((now, len(self.packet_queue)))
         if wid>0:
             self.statistics.damage_packet_qpixels.append(
@@ -351,14 +345,17 @@ class ClientConnection(StubSourceMixin):
     ######################################################################
     # info:
     def get_info(self) -> dict:
+        if not FULL_INFO:
+            return {"protocol" : "xpra"}
         info = {
                 "protocol"          : "xpra",
                 "connection_time"   : int(self.connection_time),
-                "elapsed_time"      : int(monotonic_time()-self.connection_time),
+                "elapsed_time"      : int(monotonic()-self.connection_time),
                 "counter"           : self.counter,
                 "hello-sent"        : self.hello_sent,
                 "jitter"            : self.jitter,
                 "adapter-type"      : self.adapter_type,
+                "ssh-auth-sock"     : self.ssh_auth_sock,
                 "bandwidth-limit"   : {
                     "detection"     : self.bandwidth_detection,
                     "actual"        : self.soft_bandwidth_limit or 0,
@@ -376,6 +373,8 @@ class ClientConnection(StubSourceMixin):
         info = {
             "lock"  : bool(self.lock),
             "share" : bool(self.share),
+            "xdg-menu" : bool(self.xdg_menu),
+            "xdg-menu-udpate" : bool(self.xdg_menu_update),
             }
         return info
 
@@ -391,7 +390,7 @@ class ClientConnection(StubSourceMixin):
 
 
     def send_server_event(self, *args):
-        if self.wants_events:
+        if "events" in self.wants:
             self.send_more("server-event", *args)
 
 
