@@ -36,12 +36,9 @@ class EncodingsMixin(StubSourceMixin):
 
     @classmethod
     def is_needed(cls, caps : typedict) -> bool:
-        return bool(caps.strtupleget("encodings"))
+        return bool(caps.strtupleget("encodings")) or caps.boolget("windows")
 
     def init_state(self):
-        self.wants_encodings = False
-        self.wants_features = False
-
         #contains default values, some of which may be supplied by the client:
         self.default_batch_config = batch_config.DamageBatchConfig()
         self.global_batch_config = self.default_batch_config.clone()      #global batch config
@@ -51,7 +48,7 @@ class EncodingsMixin(StubSourceMixin):
         self.encodings = ()                         #all the encodings supported by the client
         self.core_encodings = ()
         self.encodings_packet = False               #supports delayed encodings initialization?
-        self.window_icon_encodings = ["premult_argb32"]
+        self.window_icon_encodings = []
         self.rgb_formats = ("RGB",)
         self.encoding_options = typedict()
         self.icons_encoding_options = typedict()
@@ -113,9 +110,9 @@ class EncodingsMixin(StubSourceMixin):
 
     def get_caps(self) -> dict:
         caps = {}
-        if self.wants_encodings and self.encoding:
+        if "encodings" in self.wants and self.encoding:
             caps["encoding"] = self.encoding
-        if self.wants_features:
+        if "features" in self.wants:
             caps.update({
                 "auto_refresh_delay"   : self.auto_refresh_delay,
                 })
@@ -261,16 +258,15 @@ class EncodingsMixin(StubSourceMixin):
 
         delay = batch_config.START_DELAY
         dbc = self.default_batch_config
-        dbc.always      = bool(batch_value("always", batch_config.ALWAYS))
-        dbc.min_delay   = batch_value("min_delay", batch_config.MIN_DELAY, 0, 1000)
-        dbc.max_delay   = batch_value("max_delay", batch_config.MAX_DELAY, 1, 15000)
-        dbc.max_events  = batch_value("max_events", batch_config.MAX_EVENTS)
-        dbc.max_pixels  = batch_value("max_pixels", batch_config.MAX_PIXELS)
-        dbc.time_unit   = batch_value("time_unit", batch_config.TIME_UNIT, 1)
-        self.vrefresh = c.intget("vrefresh", -1)
-        dbc.match_vrefresh(self.vrefresh)
+        dbc.always      = bool(batch_value("always", dbc.always))
+        dbc.min_delay   = batch_value("min_delay", dbc.min_delay, 0, 1000)
+        dbc.max_delay   = batch_value("max_delay", dbc.max_delay, 1, 15000)
+        dbc.max_events  = batch_value("max_events", dbc.max_events)
+        dbc.max_pixels  = batch_value("max_pixels", dbc.max_pixels)
+        dbc.time_unit   = batch_value("time_unit", dbc.time_unit, 1)
         dbc.delay       = batch_value("delay", delay, dbc.min_delay)
         log("default batch config: %s", dbc)
+        self.vrefresh = c.intget("vrefresh", -1)
 
         #encodings:
         self.encodings_packet = c.boolget("encodings.packet", False)
@@ -282,7 +278,7 @@ class EncodingsMixin(StubSourceMixin):
         send_ui = getattr(self, "ui_client", True) and getattr(self, "send_windows", True)
         if send_ui and not self.core_encodings:
             raise ClientException("client failed to specify any supported encodings")
-        self.window_icon_encodings = c.strtupleget("encodings.window-icon", ("premult_argb32",))
+        self.window_icon_encodings = c.strtupleget("encodings.window-icon", ())
         #try both spellings for older versions:
         for x in ("encodings", "encoding",):
             self.rgb_formats = c.strtupleget(x+".rgb_formats", self.rgb_formats)
@@ -506,6 +502,10 @@ class EncodingsMixin(StubSourceMixin):
         for ws in tuple(self.all_window_sources()):
             ws.set_min_quality(min_quality)
 
+    def set_max_quality(self, max_quality : int):
+        for ws in tuple(self.all_window_sources()):
+            ws.set_max_quality(max_quality)
+
     def set_quality(self, quality : int):
         for ws in tuple(self.all_window_sources()):
             ws.set_quality(quality)
@@ -514,24 +514,15 @@ class EncodingsMixin(StubSourceMixin):
         for ws in tuple(self.all_window_sources()):
             ws.set_min_speed(min_speed)
 
+    def set_max_speed(self, max_speed : int):
+        for ws in tuple(self.all_window_sources()):
+            ws.set_max_speed(max_speed)
+
     def set_speed(self, speed : int):
         for ws in tuple(self.all_window_sources()):
             ws.set_speed(speed)
 
 
-    def update_batch(self, wid : int, window, batch_props):
-        ws = self.window_sources.get(wid)
-        if ws:
-            if "reset" in batch_props:
-                ws.batch_config = self.make_batch_config(wid, window)
-            for x in ("always", "locked"):
-                if x in batch_props:
-                    setattr(ws.batch_config, x, batch_props.boolget(x))
-            for x in ("min_delay", "max_delay", "timeout_delay", "delay"):
-                if x in batch_props:
-                    setattr(ws.batch_config, x, batch_props.intget(x))
-            log("batch config updated for window %s: %s", wid, ws.batch_config)
-
     def make_batch_config(self, wid : int, window):
         config = self.default_batch_config.clone()
         config.wid = wid
