@@ -1,12 +1,12 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2013-2017 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2013-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import math
+from time import monotonic
 
-from xpra.os_util import monotonic_time
 from xpra.util import envint, envbool
 from xpra.rectangle import rectangle, add_rectangle, remove_rectangle, merge_all    #@UnresolvedImport
 from xpra.log import Logger
@@ -54,7 +54,6 @@ def scoreinout(ww, wh, region, incount, outcount):
 
 
 class VideoSubregion:
-
     def __init__(self, timeout_add, source_remove, refresh_cb, auto_refresh_delay, supported=False):
         self.timeout_add = timeout_add
         self.source_remove = source_remove
@@ -82,7 +81,7 @@ class VideoSubregion:
         self.nonvideo_refresh_timer = 0
         #keep track of how much extra we batch non-video regions (milliseconds):
         self.non_max_wait = 150
-        self.min_time = monotonic_time()
+        self.min_time = monotonic()
 
     def reset(self):
         self.cancel_refresh_timer()
@@ -203,9 +202,8 @@ class VideoSubregion:
         nonvideo = []
         for r in self.refresh_regions:
             if not rect.contains_rect(r):
-                nonvideo += r.substract_rect(rect)
+                nonvideo += r.subtract_rect(rect)
         delay = max(150, self.auto_refresh_delay)
-        refreshlog("add_video_refresh(%s) rectangle=%s, delay=%ims", region, rect, delay)
         self.nonvideo_regions += nonvideo
         if self.nonvideo_regions:
             if not self.nonvideo_refresh_timer:
@@ -214,7 +212,9 @@ class VideoSubregion:
             #only keep the regions still in the video region:
             inrect = (rect.intersection_rect(r) for r in self.refresh_regions)
             self.refresh_regions = [r for r in inrect if r is not None]
-        #re-schedule the video region refresh (if we have regions to fresh):
+        refreshlog("add_video_refresh(%s) rectangle=%s, delay=%ims, nonvideo=%s, refresh_regions=%s",
+                   region, rect, delay, self.nonvideo_regions, self.refresh_regions)
+        #re-schedule the video region refresh (if we still have regions to fresh):
         if self.refresh_regions:
             self.refresh_timer = self.timeout_add(delay, self.refresh)
 
@@ -239,11 +239,12 @@ class VideoSubregion:
         #from add_video_refresh()
 
     def refresh(self):
-        refreshlog("refresh() refresh_timer=%s", self.refresh_timer)
-        #runs via timeout_add, safe to call UI!
-        self.refresh_timer = 0
         regions = self.refresh_regions
         rect = self.rectangle
+        refreshlog("refresh() refresh_timer=%s, refresh_regions=%s, rectangle=%s",
+                   self.refresh_timer, regions, rect)
+        #runs via timeout_add, safe to call UI!
+        self.refresh_timer = 0
         if rect and len(regions)>=2:
             #figure out if it makes sense to refresh the whole area,
             #or if we just send the list of smaller rectangles:
@@ -282,7 +283,7 @@ class VideoSubregion:
                             ex = max(0, ww-ew)
                         if ey<0:
                             ey = max(0, wh-eh)
-                    new_rects += r.substract(ex, ey, ew, eh)
+                    new_rects += r.subtract(ex, ey, ew, eh)
                 rects = new_rects
         return rects
 
@@ -294,8 +295,8 @@ class VideoSubregion:
             if not self.rectangle:
                 return
             #just update the fps:
-            from_time = max(starting_at, monotonic_time()-MAX_TIME, self.min_time)
-            self.time = monotonic_time()
+            from_time = max(starting_at, monotonic()-MAX_TIME, self.min_time)
+            self.time = monotonic()
             lde = tuple(x for x in tuple(last_damage_events) if x[0]>=from_time)
             incount = 0
             for _,x,y,w,h in lde:
@@ -303,7 +304,7 @@ class VideoSubregion:
                 inregion = r.intersection_rect(self.rectangle)
                 if inregion:
                     incount += inregion.width*inregion.height
-            elapsed = monotonic_time()-from_time
+            elapsed = monotonic()-from_time
             if elapsed<=0:
                 self.fps = 0
             else:
@@ -336,11 +337,11 @@ class VideoSubregion:
 
         def update_markers():
             self.counter = damage_events_count
-            self.time = monotonic_time()
+            self.time = monotonic()
 
         if self.counter+10>damage_events_count:
             #less than 10 events since last time we called update_markers:
-            elapsed = monotonic_time()-self.time
+            elapsed = monotonic()-self.time
             #how many damage events occurred since we chose this region:
             event_count = max(0, damage_events_count - self.set_at)
             #make the timeout longer when the region has worked longer:
@@ -353,7 +354,7 @@ class VideoSubregion:
                   event_count, self.counter, damage_events_count)
             return
 
-        from_time = max(starting_at, monotonic_time()-MAX_TIME, self.min_time)
+        from_time = max(starting_at, monotonic()-MAX_TIME, self.min_time)
         #create a list (copy) to work on:
         lde = tuple(x for x in tuple(last_damage_events) if x[0]>=from_time)
         dc = len(lde)
@@ -384,7 +385,7 @@ class VideoSubregion:
                 inregion = r.intersection_rect(region)
                 if inregion:
                     incount += inregion.width*inregion.height*int(count)
-                outregions = r.substract_rect(region)
+                outregions = r.subtract_rect(region)
                 for x in outregions:
                     if ignore_size>0 and x.width*x.height<ignore_size:
                         #skip small region outside rectangle
@@ -400,7 +401,7 @@ class VideoSubregion:
                 r = rectangle(x,y,w,h)
                 new_rects = []
                 for cr in rects:
-                    new_rects += cr.substract_rect(r)
+                    new_rects += cr.subtract_rect(r)
                 if not new_rects:
                     #nothing left: damage covered the whole rect
                     return 1.0
@@ -440,17 +441,18 @@ class VideoSubregion:
                     d_ratio = damaged_ratio(region)
                 score = int(score * math.sqrt(d_ratio))
                 children_boost = int(region in children_rects)*SUBWINDOW_REGION_BOOST
-            sslog("testing %12s video region %34s: %3i%% in, %3i%% out, %3i%% of window, damaged ratio=%.2f, children_boost=%i, score=%2i",
+            sslog("testing %12s video region %34s: "
+                  "%3i%% in, %3i%% out, %3i%% of window, damaged ratio=%.2f, children_boost=%i, score=%2i",
                   info, region, ipct, opct, 100*region.width*region.height/ww/wh, d_ratio, children_boost, score)
             scores[region] = score
             return score
 
         def updateregion(rect):
             self.rectangle = rect
-            self.time = monotonic_time()
+            self.time = monotonic()
             self.inout = inoutcount(rect)
             self.score = scoreinout(ww, wh, rect, *self.inout)
-            elapsed = monotonic_time()-from_time
+            elapsed = monotonic()-from_time
             if elapsed<=0:
                 self.fps = 0
             else:
