@@ -1,16 +1,15 @@
 # This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2021 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from gi.repository import GObject, Gtk, Gdk
+from gi.repository import GObject, Gtk
 
 from xpra.util import envint, envbool, typedict
 from xpra.common import MAX_WINDOW_SIZE
-from xpra.gtk_common.gobject_util import one_arg_signal, non_none_list_accumulator, SIGNAL_RUN_LAST
-from xpra.gtk_common.error import XError, XSwallowContext, xlog
-from xpra.x11.gtk_x11 import GDKX11Window
+from xpra.gtk_common.gobject_util import one_arg_signal
+from xpra.gtk_common.error import XError, xsync, xswallow, xlog
 from xpra.x11.gtk_x11.send_wm import send_wm_take_focus
 from xpra.x11.gtk_x11.prop import prop_set, prop_get
 from xpra.x11.prop_conv import MotifWMHints
@@ -18,12 +17,13 @@ from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImpo
 from xpra.x11.common import Unmanageable
 from xpra.x11.models.size_hints_util import sanitize_size_hints
 from xpra.x11.models.base import BaseWindowModel, constants
-from xpra.x11.models.core import sanestr, xswallow, xsync
+from xpra.x11.models.core import sanestr
 from xpra.x11.gtk_x11.gdk_bindings import (
     add_event_receiver, remove_event_receiver,
     get_children,
     calc_constrained_size,
     x11_get_server_time,
+    get_pywindow,
     )
 
 from xpra.gtk_common.gtk_util import get_default_root_window
@@ -84,9 +84,22 @@ class WindowModel(BaseWindowModel):
 
     __gproperties__ = dict(BaseWindowModel.__common_properties__)
     __gproperties__.update({
-        "owner": (GObject.TYPE_PYOBJECT,
-                  "Owner", "",
-                  GObject.ParamFlags.READABLE),
+        # Client(s) state, is this window shown somewhere?
+        # (then it can't be iconified)
+        "shown": (GObject.TYPE_BOOLEAN,
+                  "Is this window shown somewhere", "",
+                  False,
+                  GObject.ParamFlags.READWRITE),
+        # To prevent resizing loops,
+        # the client must provide this counter when resizing the window
+        # (and if the counter has been incremented since, we ignore the request)
+        "resize-counter": (GObject.TYPE_INT64,
+                           "", "",
+                           0, GObject.G_MAXINT64, 1,
+                           GObject.ParamFlags.READWRITE),
+        "client-geometry": (GObject.TYPE_PYOBJECT,
+                            "The current window geometry used by client(s)", "",
+                            GObject.ParamFlags.READWRITE),
         # Interesting properties of the client window, that will be
         # automatically kept up to date:
         "requested-position": (GObject.TYPE_PYOBJECT,
@@ -130,7 +143,6 @@ class WindowModel(BaseWindowModel):
         })
     __gsignals__ = dict(BaseWindowModel.__common_signals__)
     __gsignals__.update({
-        "ownership-election"            : (SIGNAL_RUN_LAST, GObject.TYPE_PYOBJECT, (), non_none_list_accumulator),
         "child-map-request-event"       : one_arg_signal,
         "child-configure-request-event" : one_arg_signal,
         "xpra-destroy-event"            : one_arg_signal,
@@ -138,7 +150,8 @@ class WindowModel(BaseWindowModel):
 
     _property_names         = BaseWindowModel._property_names + [
                               "size-hints", "icon-title", "icons", "decorations",
-                              "modal", "set-initial-position", "iconic",
+                              "modal", "set-initial-position", "requested-position",
+                              "iconic",
                               ]
     _dynamic_property_names = BaseWindowModel._dynamic_property_names + [
                               "size-hints", "icon-title", "icons", "decorations", "modal", "iconic"]
@@ -182,13 +195,10 @@ class WindowModel(BaseWindowModel):
         # clamp this window to the desktop size:
         x, y = self._clamp_to_desktop(ox, oy, ow, oh)
         geomlog("setup() clamp_to_desktop(%s)=%s", ogeom, (x, y))
-        self.corral_window = GDKX11Window(self.parking_window,
-                                        x=x, y=y, width=ow, height=oh,
-                                        window_type=Gdk.WindowType.CHILD,
-                                        event_mask=Gdk.EventMask.PROPERTY_CHANGE_MASK,
-                                        title = "CorralWindow-%#x" % self.xid)
-        cxid = self.corral_window.get_xid()
-        log("setup() corral_window=%#x", cxid)
+        parking_xid = self.parking_window.get_xid()
+        cxid = X11Window.CreateCorralWindow(parking_xid, self.xid, x, y)
+        self.corral_window = get_pywindow(cxid)
+        log("setup() corral_xid=%#x, corral_window=", cxid, self.corral_window)
         prop_set(self.corral_window, "_NET_WM_NAME", "utf8", "Xpra-CorralWindow-%#x" % self.xid)
         X11Window.substructureRedirect(cxid)
         add_event_receiver(self.corral_window, self)
@@ -244,6 +254,9 @@ class WindowModel(BaseWindowModel):
         #this is here to trigger X11 errors if any are pending
         #or if the window is deleted already:
         self.client_window.get_geometry()
+        self._internal_set_property("shown", False)
+        self._internal_set_property("resize-counter", 0)
+        self._internal_set_property("client-geometry", None)
 
 
     def _clamp_to_desktop(self, x, y, w, h):
@@ -330,7 +343,6 @@ class WindowModel(BaseWindowModel):
 
     def do_unmanaged(self, wm_exiting):
         log("unmanaging window: %s (%s - %s)", self, self.corral_window, self.client_window)
-        self._internal_set_property("owner", None)
         cwin = self.corral_window
         if cwin:
             self.corral_window = None
@@ -338,7 +350,7 @@ class WindowModel(BaseWindowModel):
             geom = None
             #use a new context so we will XSync right here
             #and detect if the window is already gone:
-            with XSwallowContext():
+            with xswallow:
                 geom = X11Window.getGeometry(self.xid)
             if geom is not None:
                 if self.client_reparented:
@@ -360,7 +372,10 @@ class WindowModel(BaseWindowModel):
             if wm_exiting:
                 self.client_window.show_unraised()
             #it is now safe to destroy the corral window:
-            cwin.destroy()
+            cxid = cwin.get_xid()
+            if cxid:
+                with xsync:
+                    X11Window.DestroyWindow(cxid)
         super().do_unmanaged(wm_exiting)
 
 
@@ -377,6 +392,7 @@ class WindowModel(BaseWindowModel):
             if X11Window.is_mapped(self.xid):
                 self.last_unmap_serial = X11Window.Unmap(self.xid)
                 log("client window %#x unmapped, serial=%#x", self.xid, self.last_unmap_serial)
+                self._internal_set_property("shown", False)
 
     def map(self):
         with xsync:
@@ -430,74 +446,53 @@ class WindowModel(BaseWindowModel):
     # Hooks for WM
     #########################################
 
-    def ownership_election(self):
-        #returns True if we have updated the geometry
-        candidates = self.emit("ownership-election")
-        if candidates:
-            rating, winner = sorted(candidates)[-1]
-            if rating < 0:
-                winner = None
-        else:
-            winner = None
-        old_owner = self.get_property("owner")
-        log("ownership_election() winner=%s, old owner=%s, candidates=%s", winner, old_owner, candidates)
-        if old_owner is winner:
-            return False
-        if old_owner is not None:
-            self.corral_window.hide()
-            self.corral_window.reparent(self.parking_window, 0, 0)
-        self._internal_set_property("owner", winner)
-        if winner is not None:
-            winner.take_window(self, self.corral_window)
-            self._update_client_geometry()
-            self.corral_window.show_unraised()
-            return True
+    def show(self):
+        self.map()
+        self._internal_set_property("shown", True)
+        if self.get_property("iconic"):
+            self.set_property("iconic", False)
+        self._update_client_geometry()
+        self.corral_window.show_unraised()
+
+    def hide(self):
+        self._internal_set_property("shown", False)
+        self.corral_window.hide()
+        self.corral_window.reparent(self.parking_window, 0, 0)
+        self.send_configure_notify()
+
+    def send_configure_notify(self):
         with xswallow:
             X11Window.sendConfigureNotify(self.xid)
-        return False
 
-    def maybe_recalculate_geometry_for(self, maybe_owner):
-        if maybe_owner and self.get_property("owner") is maybe_owner:
-            self._update_client_geometry()
 
     def _update_client_geometry(self):
         """ figure out where we're supposed to get the window geometry from,
             and call do_update_client_geometry which will send a Configure and Notify
         """
-        owner = self.get_property("owner")
-        if owner is not None:
-            geomlog("_update_client_geometry: using owner=%s (setup_done=%s)", owner, self._setup_done)
-            def window_size():
-                return  owner.window_size(self)
-            def window_position(w, h):
-                return  owner.window_position(self, w, h)
+        geometry = self.get_property("client-geometry")
+        if geometry is not None:
+            geomlog("_update_client_geometry: using client-geometry=%s", geometry)
         elif not self._setup_done:
             #try to honour initial size and position requests during setup:
-            def window_size():
-                return self.get_property("requested-size")
-            def window_position(_w, _h):
-                return self.get_property("requested-position")
-            geomlog("_update_client_geometry: using initial size=%s and position=%s", window_size, window_position)
+            w, h = self.get_property("requested-size")
+            x, y = self.get_property("requested-position")
+            geometry = x, y, w, h
+            geomlog("_update_client_geometry: using initial geometry=%s", geometry)
         else:
-            geomlog("_update_client_geometry: ignored, owner=%s, setup_done=%s", owner, self._setup_done)
-            def window_size():
-                return self.get_property("geometry")[2:4]
-            def window_position(_w, _h):
-                return self.get_property("geometry")[:2]
-        self._do_update_client_geometry(window_size, window_position)
+            geometry = self.get_property("geometry")
+            geomlog("_update_client_geometry: using current geometry=%s", geometry)
+        self._do_update_client_geometry(geometry)
 
 
-    def _do_update_client_geometry(self, window_size_cb, window_position_cb):
-        allocated_w, allocated_h = window_size_cb()
-        geomlog("_do_update_client_geometry: allocated %ix%i (from %s)", allocated_w, allocated_h, window_size_cb)
+    def _do_update_client_geometry(self, geometry):
+        geomlog("_do_update_client_geometry(%s)", geometry)
         hints = self.get_property("size-hints")
+        x, y, allocated_w, allocated_h = geometry
         w, h = self.calc_constrained_size(allocated_w, allocated_h, hints)
         geomlog("_do_update_client_geometry: size(%s)=%ix%i", hints, w, h)
-        x, y = window_position_cb(w, h)
-        geomlog("_do_update_client_geometry: position=%ix%i (from %s)", x, y, window_position_cb)
         self.corral_window.move_resize(x, y, w, h)
         self._updateprop("geometry", (x, y, w, h))
-        with xswallow:
+        with xlog:
             X11Window.configureAndNotify(self.xid, 0, 0, w, h)
 
     def do_xpra_configure_event(self, event):
@@ -564,9 +559,12 @@ class WindowModel(BaseWindowModel):
         moved = x!=0 or y!=0
         geomlog("resize_corral_window%s hints=%s, constrained size=%s, geometry=%s, resized=%s, moved=%s",
                 (x, y, w, h), hints, (w, h), (cx, cy, cw, ch), resized, moved)
+        if moved:
+            self._internal_set_property("requested-position", (x, y))
+            self._internal_set_property("set-initial-position", True)
+
         if resized:
             if moved:
-                self._internal_set_property("set-initial-position", True)
                 geomlog("resize_corral_window() move and resize from %s to %s", (cox, coy, cow, coh), (x, y, w, h))
                 self.corral_window.move_resize(x, y, w, h)
                 self.client_window.move(0, 0)
@@ -576,7 +574,6 @@ class WindowModel(BaseWindowModel):
                 self.corral_window.resize(w, h)
                 self._updateprop("geometry", (cx, cy, w, h))
         elif moved:
-            self._internal_set_property("set-initial-position", True)
             geomlog("resize_corral_window() moving corral window from %s to %s", (cox, coy), (x, y))
             self.corral_window.move(x, y)
             self.client_window.move(0, 0)
@@ -601,7 +598,7 @@ class WindowModel(BaseWindowModel):
             ry = y
         if event.value_mask & CWX or event.value_mask & CWY:
             self._internal_set_property("set-initial-position", True)
-            self._updateprop("requested-position", (rx, ry))
+            self._internal_set_property("requested-position", (rx, ry))
 
         rw, rh = self.get_property("requested-size")
         if event.value_mask & CWWidth:
@@ -611,7 +608,7 @@ class WindowModel(BaseWindowModel):
             h = event.height
             rh = h
         if event.value_mask & CWWidth or event.value_mask & CWHeight:
-            self._updateprop("requested-size", (rw, rh))
+            self._internal_set_property("requested-size", (rw, rh))
 
         if event.value_mask & CWStackMode:
             self.emit("restack", event.detail, event.above)
@@ -643,7 +640,9 @@ class WindowModel(BaseWindowModel):
                 w = event.data[3]
             if event.data[0] & 0x800:
                 h = event.data[4]
-            self._internal_set_property("set-initial-position", (event.data[0] & 0x100) or (event.data[0] & 0x200))
+            if (event.data[0] & 0x100) or (event.data[0] & 0x200):
+                self._internal_set_property("set-initial-position", True)
+                self._internal_set_property("requested-position", (x, y))
             #honour hints:
             hints = self.get_property("size-hints")
             w, h = self.calc_constrained_size(w, h, hints)
@@ -670,7 +669,7 @@ class WindowModel(BaseWindowModel):
             geomlog("update_size_constraints%s less restrictive, no need to recalculate", (minw, minh, maxw, maxh))
             return
         geomlog("update_size_constraints%s recalculating client geometry", (minw, minh, maxw, maxh))
-        if self._setup_done and self.get_property("owner"):
+        if self.get_property("shown"):
             self._update_client_geometry()
 
     #########################################
@@ -760,7 +759,7 @@ class WindowModel(BaseWindowModel):
         # reduces the chance for us to get caught in loops:
         if self._updateprop("size-hints", hints):
             metalog("updated: size-hints=%s", hints)
-            if self._setup_done and self.get_property("owner"):
+            if self._setup_done and self.get_property("shown"):
                 self._update_client_geometry()
 
 
@@ -788,16 +787,16 @@ class WindowModel(BaseWindowModel):
         wmclass_name = c_i[0]
         if not wmclass_name:
             return None
-        it = Gtk.IconTheme.get_default()
+        it = Gtk.IconTheme.get_default()  # pylint: disable=no-member
         pixbuf = None
         iconlog("get_default_window_icon(%i) icon theme=%s, wmclass_name=%s", size, it, wmclass_name)
         for icon_name in (
-            "%s-color" % wmclass_name,
+            f"{wmclass_name}-color",
             wmclass_name,
-            "%s_%ix%i" % (wmclass_name, size, size),
-            "application-x-%s" % wmclass_name,
-            "%s-symbolic" % wmclass_name,
-            "%s.symbolic" % wmclass_name,
+            f"{wmclass_name}_{size}x{size}",
+            f"application-x-{wmclass_name}",
+            f"{wmclass_name}-symbolic",
+            f"{wmclass_name}.symbolic",
             ):
             i = it.lookup_icon(icon_name, size, 0)
             iconlog("lookup_icon(%s)=%s", icon_name, i)
